@@ -513,8 +513,7 @@ injectIntoScope <- function(x, ...) {
   x
 }
 
-### S3/S4 combo for as.data.frame.DataFrame
-as.data.frame.DataFrame <- function(x, row.names=NULL, optional=FALSE, ...)
+.as.data.frame.DataFrame <- function(x, row.names=NULL, optional=FALSE, ...)
 {
     if (length(list(...)))
         warning("Arguments in '...' ignored")
@@ -536,7 +535,7 @@ as.data.frame.DataFrame <- function(x, row.names=NULL, optional=FALSE, ...)
             c(l, list(row.names=row.names),
               check.names=!optional, stringsAsFactors=FALSE))
 }
-setMethod("as.data.frame", "DataFrame", as.data.frame.DataFrame)
+setMethod("as.data.frame", "DataFrame", .as.data.frame.DataFrame)
 
 setMethod("as.matrix", "DataFrame", function(x) {
   if (length(x) == 0L)
@@ -629,6 +628,8 @@ setAs("AsIs", "DataFrame",
 
 setAs("ANY", "AsIs", function(from) I(from))
 
+setAs("ANY", "DataTableORNULL", function(from) as(from, "DataFrame"))
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Combining.
 ###
@@ -696,7 +697,8 @@ setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
       combined
     })
     names(cols) <- colnames(df)
-    ans <- new("DataFrame", listData = cols, nrows = NROW(cols[[1]]))
+    ans <- new2("DataFrame", listData = cols, nrows = NROW(cols[[1]]),
+                             check = FALSE)
   }
 
   rn <- unlist(lapply(args, rownames), use.names=FALSE)
