@@ -3,7 +3,7 @@
 ### -------------------------------------------------------------------------
 ###
 ### List objects are Vector objects with "[[", "elementType" and
-### "elementLengths" methods.
+### "elementNROWS" methods.
 ###
 
 setClass("List",
@@ -24,11 +24,18 @@ setGeneric("elementType", function(x, ...) standardGeneric("elementType"))
 setMethod("elementType", "List", function(x) x@elementType)
 setMethod("elementType", "vector", function(x) storage.mode(x))
 
-setGeneric("elementLengths", function(x) standardGeneric("elementLengths"))
+setGeneric("elementNROWS", function(x) standardGeneric("elementNROWS"))
 
-setMethod("elementLengths", "ANY", sapply_NROW)
+setMethod("elementNROWS", "ANY", sapply_NROW)
 
-setMethod("elementLengths", "List",
+### Used in the SGSeq package!
+quick_togroup <- function(x)
+{
+    x_eltNROWS <- elementNROWS(x)
+    rep.int(seq_along(x_eltNROWS), x_eltNROWS)
+}
+
+setMethod("elementNROWS", "List",
     function(x)
     {
         y <- as.list(x)
@@ -39,11 +46,23 @@ setMethod("elementLengths", "List",
             return(ans)
         }
         if (length(dim(y[[1L]])) < 2L)
-            return(elementLengths(y))
+            return(elementNROWS(y))
         return(sapply(y, NROW))
     }
 )
 
+### In BioC 3.3 elementLengths() was renamed elementNROWS() (the old name was
+### clearly a misnomer). For backward compatibility the old name was kept
+### around, deprecated, and became an "alias" for elementNROWS().
+setGeneric("elementLengths", function(x) standardGeneric("elementLengths"))
+setMethod("elementLengths", "ANY",
+    function(x)
+    {
+        .Deprecated("elementNROWS")
+        elementNROWS(x)
+    }
+)
+
 setGeneric("isEmpty", function(x) standardGeneric("isEmpty"))
 setMethod("isEmpty", "ANY",
           function(x)
@@ -59,7 +78,37 @@ setMethod("isEmpty", "ANY",
               sapply(x, function(xx) all(isEmpty(xx)))
           })
 ### A List object is considered empty iff all its elements are empty.
-setMethod("isEmpty", "List", function(x) all(elementLengths(x) == 0L))
+setMethod("isEmpty", "List", function(x) all(elementNROWS(x) == 0L))
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### relistToClass()
+###
+### 'relistToClass(x)' is the opposite of 'elementType(y)' in the sense that
+### the former returns the class of the result of relisting (or splitting)
+### 'x' while the latter returns the class of the result of unlisting (or
+### unsplitting) 'y'.
+###
+### More formally, if 'x' is an object that is relistable and 'y' a list-like
+### object:
+###    relistToClass(x) == class(relist(x, some_skeleton))
+###    elementType(y) == class(unlist(y))
+###
+### As a consequence, for any object 'x' for which relistToClass() is defined
+### and returns a valid class, 'elementType(new(relistToClass(x)))' should
+### return 'class(x)'.
+###
+
+setGeneric("relistToClass", function(x) standardGeneric("relistToClass"))
+
+.selectListClassName <- function(x) {
+  cn <- listClassName("Compressed", x)
+  if (cn == "CompressedList")
+    cn <- listClassName("Simple", x)
+  cn
+}
+
+setMethod("relistToClass", "ANY", function(x) .selectListClassName(class(x)))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -90,6 +139,69 @@ setMethod("show", "List",
           })
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### unlist()
+###
+
+### NOT exported. Assumes 'names1' is not NULL.
+make_unlist_result_names <- function(names1, names2)
+{
+    if (is.null(names2))
+        return(names1)
+    idx2 <- names2 != "" | is.na(names2)
+    idx1 <- names1 != "" | is.na(names1)
+    idx <- idx1 & idx2
+    if (any(idx))
+        names1[idx] <- paste(names1[idx], names2[idx], sep = ".")
+    idx <- !idx1 & idx2
+    if (any(idx))
+        names1[idx] <- names2[idx]
+    names1
+}
+
+setMethod("unlist", "List",
+    function(x, recursive=TRUE, use.names=TRUE)
+    {
+        if (!identical(recursive, TRUE))
+            stop("\"unlist\" method for List objects ",
+                 "does not support the 'recursive' argument")
+        if (!isTRUEorFALSE(use.names))
+            stop("'use.names' must be TRUE or FALSE")
+        if (length(x) == 0L)
+            return(NULL)
+        x_names <- names(x)
+        if (!is.null(x_names))
+            names(x) <- NULL
+        xx <- as.list(x)
+        if (length(dim(xx[[1L]])) < 2L) {
+            ans <- do.call(c, xx)
+            ans_names0 <- names(ans)
+            if (use.names) {
+                if (!is.null(x_names)) {
+                    ans_names <- rep.int(x_names, elementNROWS(x))
+                    ans_names <- make_unlist_result_names(ans_names, ans_names0)
+                    try_result <- try(names(ans) <- ans_names, silent=TRUE)
+                    if (inherits(try_result, "try-error"))
+                        warning("failed to set names on the result ",
+                                "of unlisting a ", class(x), " object")
+                }
+            } else {
+                ## This is consistent with base::unlist but is not consistent
+                ## with unlist,CompressedList. See comments and FIXME note in
+                ## the unlist,CompressedList code for more details.
+                if (!is.null(ans_names0))
+                    names(ans) <- NULL
+            }
+        } else {
+            ans <- do.call(rbind, xx)
+            if (!use.names)
+                rownames(ans) <- NULL
+        }
+        ans
+    }
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
 ###
@@ -104,9 +216,9 @@ setMethod("show", "List",
         unlisted_i <- as.integer(unlisted_i)
     if (anyMissingOrOutside(unlisted_i, lower=1L))
         return(FALSE)
-    x_eltlens <- elementLengths(x)
-    i_eltlens <- elementLengths(i)
-    if (any(unlisted_i > rep.int(x_eltlens, i_eltlens)))
+    x_eltNROWS <- elementNROWS(x)
+    i_eltNROWS <- elementNROWS(i)
+    if (any(unlisted_i > rep.int(x_eltNROWS, i_eltNROWS)))
         return(FALSE)
     return(TRUE)
 }
@@ -154,14 +266,14 @@ setMethod("show", "List",
 ### corresponding element in 'x'.
 .adjust_elt_lengths <- function(i, x)
 {
-    x_eltlens <- unname(elementLengths(x))
-    i_eltlens <- unname(elementLengths(i))
-    idx <- which(x_eltlens != i_eltlens)
+    x_eltNROWS <- unname(elementNROWS(x))
+    i_eltNROWS <- unname(elementNROWS(i))
+    idx <- which(x_eltNROWS != i_eltNROWS)
     ## FIXME: This is rough and doesn't follow exactly the truncate-or-recycle
     ## semantic of normalizeSingleBracketSubscript() on a logical vector or
     ## logical-Rle object.
     for (k in idx)
-        i[[k]] <- rep(i[[k]], length.out=x_eltlens[k])
+        i[[k]] <- rep(i[[k]], length.out=x_eltNROWS[k])
     return(i)
 }
 
@@ -177,7 +289,7 @@ setMethod("show", "List",
 ### and 'i' is a List of numeric vectors or numeric-Rle objects.
 .unlist_NL_subscript <- function(i, x)
 {
-    offsets <- c(0L, end(PartitioningByEnd(x))[-length(x)])
+    offsets <- c(0L, end(IRanges::PartitioningByEnd(x))[-length(x)])
     i <- i + offsets
     unlist(i, use.names=FALSE)
 }
@@ -187,8 +299,8 @@ setMethod("show", "List",
 .unlist_RL_subscript <- function(i, x)
 {
     unlisted_i <- unlist(i, use.names=FALSE)
-    offsets <- c(0L, end(PartitioningByEnd(x))[-length(x)])
-    shift(unlisted_i, shift=rep.int(offsets, elementLengths(i)))
+    offsets <- c(0L, end(IRanges::PartitioningByEnd(x))[-length(x)])
+    IRanges::shift(unlisted_i, shift=rep.int(offsets, elementNROWS(i)))
 }
 
 ### Fast subset by List of logical vectors or logical-Rle objects.
@@ -203,9 +315,9 @@ setMethod("show", "List",
     unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
 
     ## Relist.
-    group <- rep.int(seq_along(x), elementLengths(x))
+    group <- rep.int(seq_along(x), elementNROWS(x))
     group <- extractROWS(group, unlisted_i)
-    ans_skeleton <- PartitioningByEnd(group, NG=length(x), names=names(x))
+    ans_skeleton <- IRanges::PartitioningByEnd(group, NG=length(x), names=names(x))
     ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
     metadata(ans) <- metadata(x)
     ans
@@ -223,8 +335,8 @@ setMethod("show", "List",
     unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
 
     ## Relist.
-    ans_breakpoints <- cumsum(unname(elementLengths(i)))
-    ans_skeleton <- PartitioningByEnd(ans_breakpoints, names=names(x))
+    ans_breakpoints <- cumsum(unname(elementNROWS(i)))
+    ans_skeleton <- IRanges::PartitioningByEnd(ans_breakpoints, names=names(x))
     ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
     metadata(ans) <- metadata(x)
     ans
@@ -243,7 +355,7 @@ setMethod("show", "List",
 
     ## Relist.
     ans_breakpoints <- cumsum(unlist(sum(width(i)), use.names=FALSE))
-    ans_skeleton <- PartitioningByEnd(ans_breakpoints, names=names(x))
+    ans_skeleton <- IRanges::PartitioningByEnd(ans_breakpoints, names=names(x))
     ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
     metadata(ans) <- metadata(x)
     ans
@@ -461,7 +573,7 @@ setReplaceMethod("$", "List",
 ### phead() and ptail(): "parallel" versions of head() and tail() for List
 ### objects. They're just fast equivalents of 'mapply(head, x, n)' and
 ### 'mapply(tail, x, n)', respectively.
-.normarg_n <- function(n, x_eltlens)
+.normarg_n <- function(n, x_eltNROWS)
 {
     if (!is.numeric(n))
         stop("'n' must be an integer vector")
@@ -469,19 +581,19 @@ setReplaceMethod("$", "List",
         n <- as.integer(n)
     if (any(is.na(n)))
         stop("'n' cannot contain NAs")
-    n <- pmin(x_eltlens, n)
+    n <- pmin(x_eltNROWS, n)
     neg_idx <- which(n < 0L)
     if (length(neg_idx) != 0L)
-        n[neg_idx] <- pmax(n[neg_idx] + x_eltlens[neg_idx], 0L)
+        n[neg_idx] <- pmax(n[neg_idx] + x_eltNROWS[neg_idx], 0L)
     n
 }
 
 phead <- function(x, n=6L)
 {
-    x_eltlens <- unname(elementLengths(x))
-    n <- .normarg_n(n, x_eltlens)
-    unlisted_i <- IRanges(start=rep.int(1L, length(n)), width=n)
-    i <- relist(unlisted_i, PartitioningByEnd(seq_along(x)))
+    x_eltNROWS <- unname(elementNROWS(x))
+    n <- .normarg_n(n, x_eltNROWS)
+    unlisted_i <- IRanges::IRanges(start=rep.int(1L, length(n)), width=n)
+    i <- relist(unlisted_i, IRanges::PartitioningByEnd(seq_along(x)))
     ans <- x[i]
     mcols(ans) <- mcols(x)
     ans
@@ -489,10 +601,10 @@ phead <- function(x, n=6L)
 
 ptail <- function(x, n=6L)
 {
-    x_eltlens <- unname(elementLengths(x))
-    n <- .normarg_n(n, x_eltlens)
-    unlisted_i <- IRanges(end=x_eltlens, width=n)
-    i <- relist(unlisted_i, PartitioningByEnd(seq_along(x)))
+    x_eltNROWS <- unname(elementNROWS(x))
+    n <- .normarg_n(n, x_eltNROWS)
+    unlisted_i <- IRanges::IRanges(end=x_eltNROWS, width=n)
+    i <- relist(unlisted_i, IRanges::PartitioningByEnd(seq_along(x)))
     ans <- x[i]
     mcols(ans) <- mcols(x)
     ans
@@ -514,9 +626,7 @@ setAs("List", "list", function(from) as.list(from))
         names(ans) <- NULL
     ans
 }
-### S3/S4 combo for as.list.List
-as.list.List <- function(x, ...) .as.list.List(x, ...)
-setMethod("as.list", "List", as.list.List)
+setMethod("as.list", "List", .as.list.List)
 
 setMethod("as.env", "List",
           function(x, enclos = parent.frame(2), tform = identity) {
@@ -554,134 +664,57 @@ listClassName <- function(impl, element.type) {
 
 setAs("ANY", "List", function(from) {
   ## since list is directed to SimpleList, we assume 'from' is non-list-like
-  relist(from, PartitioningByEnd(seq_along(from), names=names(from)))
+  relist(from, IRanges::PartitioningByEnd(seq_along(from), names=names(from)))
 })
 
 ## Special cased, because integer extends ANY (somehow) and numeric,
 ## so ambiguities are introduced due to method caching.
 setAs("integer", "List", getMethod(coerce, c("ANY", "List")))
 
-### NOT exported. Assumes 'names1' is not NULL.
-make_unlist_result_names <- function(names1, names2)
+.make_group_and_group_name <- function(x_eltNROWS, group_name.as.factor=FALSE)
 {
-    if (is.null(names2))
-        return(names1)
-    idx2 <- names2 != "" | is.na(names2)
-    idx1 <- names1 != "" | is.na(names1)
-    idx <- idx1 & idx2
-    if (any(idx))
-        names1[idx] <- paste(names1[idx], names2[idx], sep = ".")
-    idx <- !idx1 & idx2
-    if (any(idx))
-        names1[idx] <- names2[idx]
-    names1
+    if (!isTRUEorFALSE(group_name.as.factor))
+        stop("'group_name.as.factor' must be TRUE or FALSE")
+    group <- rep.int(seq_along(x_eltNROWS), x_eltNROWS)
+    x_names <- names(x_eltNROWS)
+    if (is.null(x_names)) {
+        group_name <- rep.int(NA_character_, length(group))
+        if (group_name.as.factor)
+            group_name <- factor(group_name, levels=character(0))
+    } else {
+        group_name <- rep.int(x_names, x_eltNROWS)
+        if (group_name.as.factor)
+            group_name <- factor(group_name, levels=unique(x_names))
+    }
+    data.frame(group=group, group_name=group_name, stringsAsFactors=FALSE)
 }
 
-setMethod("unlist", "List",
-    function(x, recursive=TRUE, use.names=TRUE)
-    {
-        if (!identical(recursive, TRUE))
-            stop("\"unlist\" method for List objects ",
-                 "does not support the 'recursive' argument")
-        if (!isTRUEorFALSE(use.names))
-            stop("'use.names' must be TRUE or FALSE")
-        if (length(x) == 0L)
-            return(NULL)
-        x_names <- names(x)
-        if (!is.null(x_names))
-            names(x) <- NULL
-        xx <- as.list(x)
-        if (length(dim(xx[[1L]])) < 2L) {
-            ans <- do.call(c, xx)
-            ans_names0 <- names(ans)
-            if (use.names) {
-                if (!is.null(x_names)) {
-                    ans_names <- rep.int(x_names, elementLengths(x))
-                    ans_names <- make_unlist_result_names(ans_names, ans_names0)
-                    try_result <- try(names(ans) <- ans_names, silent=TRUE)
-                    if (inherits(try_result, "try-error"))
-                        warning("failed to set names on the result ",
-                                "of unlisting a ", class(x), " object")
-                }
-            } else {
-                ## This is consistent with base::unlist but is not consistent
-                ## with unlist,CompressedList. See comments and FIXME note in
-                ## the unlist,CompressedList code for more details.
-                if (!is.null(ans_names0))
-                    names(ans) <- NULL
-            }
-        } else {
-            ans <- do.call(rbind, xx)
-            if (!use.names)
-                rownames(ans) <- NULL
-        }
-        ans
-    }
-)
-
-### S3/S4 combo for as.data.frame.List
-as.data.frame.List <- 
+.as.data.frame.List <- 
     function(x, row.names=NULL, optional=FALSE, ..., value.name="value",
              use.outer.mcols=FALSE, group_name.as.factor=FALSE)
 {
-    if (!requireNamespace("IRanges", quietly=TRUE))
-        stop("Couldn't load the IRanges package. You need to install ",
-             "the IRanges\n  package to coerce a List object to data.frame.")
-    if (!length(IRanges::togroup(x)))
-        return(data.frame())
     if (!isSingleString(value.name))
         stop("'value.name' must be a single string")
     if (!isTRUEorFALSE(use.outer.mcols))
         stop("'use.outer.mcols' must be TRUE or FALSE")
-    if (!isTRUEorFALSE(group_name.as.factor))
-        stop("'group_name.as.factor' must be TRUE or FALSE")
-    if (!(is.null(row.names) || is.character(row.names)))
-        stop("'row.names'  must be NULL or a character vector")
-
-    if (!length(group_name <- names(x)[IRanges::togroup(x)]))
-        group_name <- NA_character_
-    if (group_name.as.factor)
-        group_name <- factor(group_name, levels=unique(group_name))
-    xx <- cbind(data.frame(group=IRanges::togroup(x), group_name, 
-                           stringsAsFactors=FALSE), 
-                as.data.frame(unlist(x, use.names=FALSE), 
-                              row.names=row.names, optional=optional, ...))
-    if (ncol(xx) == 3)
-        colnames(xx)[3] <- value.name
-    if (use.outer.mcols)
-        if (length(md <- mcols(x)[IRanges::togroup(x), , drop=FALSE]))
-            return(cbind(xx, md))
-
-    xx
+    ans <- as.data.frame(unlist(x, use.names=FALSE),
+                         row.names=row.names, optional=optional, ...)
+    if (ncol(ans) == 1L)
+        colnames(ans)[1L] <- value.name
+    group_and_group_name <- .make_group_and_group_name(elementNROWS(x),
+                                                       group_name.as.factor)
+    ans <- cbind(group_and_group_name, ans)
+    if (use.outer.mcols) {
+        x_mcols <- mcols(x)
+        if (length(x_mcols) != 0L) {
+            extra_cols <- as.data.frame(x_mcols)
+            extra_cols <- extract_data_frame_rows(extra_cols, ans[[1L]])
+            ans <- cbind(ans, extra_cols)
+        }
+    }
+    ans
 }
-setMethod("as.data.frame", "List", as.data.frame.List)
+setMethod("as.data.frame", "List", .as.data.frame.List)
 
 setAs("List", "data.frame", function(from) as.data.frame(from))
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Element-wise appending for list-like objects
-###
-### TODO: Move to List-utils.R (Looping methods section).
-###
-pc <- function(...) {
-  args <- list(...)
-  args <- Filter(Negate(is.null), args)
-  if (length(args) <= 1L) {
-    return(args[[1L]])
-  }
-  if (length(unique(elementLengths(args))) > 1L) {
-    stop("All arguments in '...' must have the same length")
-  }
-
-  ans_unlisted <- do.call(c, lapply(args, unlist, use.names=FALSE))
-  ans_group <- structure(do.call(c, lapply(args, togroup)),
-                         class="factor",
-                         levels=as.character(seq_along(args[[1L]])))
-  
-  ans <- splitAsList(ans_unlisted, ans_group)
-
-  names(ans) <- names(args[[1L]])
-  ans
-}
-
