@@ -2,8 +2,6 @@
 ### Rle objects
 ### -------------------------------------------------------------------------
 ###
-### Class definitions
-###
 
 setClass("Rle",
          representation(values = "vectorORfactor",
@@ -25,15 +23,18 @@ setClass("Rle",
              #    msg <- c(msg, "consecutive runs must have different values")
              if (is.null(msg)) TRUE else msg
          })
+
  
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Accessor methods.
+### Getters
 ###
 
 setGeneric("runLength", signature = "x",
            function(x) standardGeneric("runLength"))
 setMethod("runLength", "Rle", function(x) x@lengths)
  
+setMethod("length", "Rle", function(x) sum(runLength(x)))
+
 setGeneric("runValue", signature = "x",
            function(x) standardGeneric("runValue"))
 setMethod("runValue", "Rle", function(x) x@values)
@@ -45,58 +46,56 @@ setMethod("start", "Rle", function(x) .Call2("Rle_start", x, PACKAGE="S4Vectors"
 setMethod("end", "Rle", function(x) .Call2("Rle_end", x, PACKAGE="S4Vectors"))
 setMethod("width", "Rle", function(x) runLength(x))
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Replace methods.
+### Constructor
+###
+
+### Low-level constructor.
+new_Rle <- function(values=logical(0), lengths=integer(0), check=TRUE)
+{
+    stopifnot(is(values, "vectorORfactor"))
+    stopifnot(is.numeric(lengths))
+    if (!is.integer(lengths))
+        lengths <- as.integer(lengths)
+    if (!isTRUEorFALSE(check))
+        stop("'check' must be TRUE or FALSE")
+    .Call2("Rle_constructor", values, lengths, check, 0L, PACKAGE="S4Vectors")
+}
+
+setGeneric("Rle", signature="values",
+    function(values=logical(0), lengths=integer(0)) standardGeneric("Rle")
+)
+
+setMethod("Rle", "ANY",
+    function(values=logical(0), lengths=integer(0)) new_Rle(values, lengths)
+)
+
+setMethod("Rle", "Rle",
+    function(values=logical(0), lengths=integer(0))
+    {
+        if (!missing(lengths))
+            stop(wmsg("'lengths' cannot be supplied when calling Rle() ",
+                      "on an Rle object"))
+        values
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Setters
 ###
 
 setGeneric("runLength<-", signature="x",
            function(x, value) standardGeneric("runLength<-"))
 setReplaceMethod("runLength", "Rle",
-                 function(x, value) Rle(values = runValue(x), lengths = value))
+                 function(x, value) Rle(runValue(x), value))
          
 setGeneric("runValue<-", signature="x",
            function(x, value) standardGeneric("runValue<-"))
 setReplaceMethod("runValue", "Rle",
-                 function(x, value) Rle(values = value, lengths = runLength(x)))
+                 function(x, value) Rle(value, runLength(x)))
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Constructors
-###
-
-setGeneric("Rle", signature = c("values", "lengths"),
-           function(values, lengths, ...) standardGeneric("Rle"))
-
-setMethod("Rle", signature = c(values = "missing", lengths = "missing"),
-          function(values, lengths)
-              new2("Rle", values = vector(), lengths = integer(), check=FALSE))
-
-setMethod("Rle", signature = c(values = "vectorORfactor", lengths = "missing"),
-          function(values, lengths) Rle(values, integer(0), check = FALSE))
-
-setMethod("Rle", signature = c(values = "vectorORfactor", lengths = "integer"),
-          function(values, lengths, check = TRUE)
-          {
-              if (!isTRUEorFALSE(check))
-                  stop("'check' must be TRUE or FALSE")
-              ans <- .Call2("Rle_constructor",
-                            values, lengths, check, 0L,
-                            PACKAGE="S4Vectors")
-              if (is.factor(values)) {
-                  ans@values <-
-                    factor(ans@values,
-                           levels = seq_len(length(levels(values))),
-                           labels = levels(values))
-              }
-              ans
-          })
-
-setMethod("Rle", signature = c(values = "vectorORfactor", lengths = "numeric"),
-          function(values, lengths, check = TRUE)
-              Rle(values = values, lengths = as.integer(lengths),
-                  check = check))
-
-setMethod("Rle", signature = c(values = "Rle", lengths = "missing"),
-          function(values, lengths, check = TRUE) values)
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion
@@ -136,60 +135,119 @@ asFactorOrFactorRle <- function(x) {
   }
 }
 
-### S3/S4 combo for as.list.Rle
 .as.list.Rle <- function(x) as.list(as.vector(x))
-as.list.Rle <- function(x, ...) .as.list.Rle(x, ...)
-setMethod("as.list", "Rle", as.list.Rle)
+setMethod("as.list", "Rle", .as.list.Rle)
+
+setGeneric("decode", function(x, ...) standardGeneric("decode"))
+setMethod("decode", "ANY", identity)
 
 decodeRle <- function(x) rep.int(runValue(x), runLength(x))
+setMethod("decode", "Rle", decodeRle)
 
-### S3/S4 combo for as.data.frame.Rle
-as.data.frame.Rle <- function(x, row.names=NULL, optional=FALSE, ...)
+.as.data.frame.Rle <- function(x, row.names=NULL, optional=FALSE, ...)
 {
     value <- decodeRle(x)
     as.data.frame(value, row.names=row.names,
                   optional=optional, ...)
 }
-setMethod("as.data.frame", "Rle", as.data.frame.Rle)
+setMethod("as.data.frame", "Rle", .as.data.frame.Rle)
 
 getStartEndRunAndOffset <- function(x, start, end) {
     .Call2("Rle_getStartEndRunAndOffset", x, start, end, PACKAGE="S4Vectors")
 }
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### General methods
+### Subsetting workhorses
+###
+### These are the low-level functions that do the real work of subsetting an
+### Rle object. The final coercion to class(x) is to make sure that they act
+### like an endomorphism on objects that belong to a subclass of Rle (the
+### VariantAnnotation package defines Rle subclasses).
+### Note that they drop the metadata columns!
 ###
 
-setMethod("extractROWS", "Rle",
+extract_range_from_Rle <- function(x, start, end)
+{
+    ans <- .Call2("Rle_extract_range", x, start, end, PACKAGE="S4Vectors")
+    as(ans, class(x))  # so the function is an endomorphism
+}
+
+.normarg_method <- function(method)
+{
+    if (!(isSingleNumber(method) && method >= 0 && method <= 3))
+        stop("'method' must be a single integer between 0 and 3")
+    if (!is.integer(method))
+        method <- as.integer(method)
+    method
+}
+
+### Used in GenomicRanges.
+map_ranges_to_runs <- function(run_lens, start, width, method=0L)
+{
+    method <- .normarg_method(method)
+    .Call2("ranges_to_runs_mapper", run_lens, start, width, method,
+                                    PACKAGE="S4Vectors")
+}
+
+### NOT exported but used in IRanges package (by "extractROWS" method with
+### signature Rle,RangesNSBS).
+extract_ranges_from_Rle <- function(x, start, width, method=0L, as.list=FALSE)
+{
+    method <- .normarg_method(method)
+    if (!isTRUEorFALSE(as.list))
+        stop("'as.list' must be TRUE or FALSE")
+    ans <- .Call2("Rle_extract_ranges", x, start, width, method, as.list,
+                                        PACKAGE="S4Vectors")
+    ## The function must act like an endomorphism.
+    x_class <- class(x)
+    if (!as.list)
+        return(as(ans, x_class))
+    ## 'ans' is a list of Rle instances.
+    if (x_class == "Rle")
+        return(ans)
+    lapply(ans, as, x_class)
+}
+
+### TODO: Optimize this, maybe by implementing a simpler version of .Call
+### entry point "Rle_extract_ranges" that takes 1 integer vector instead of 2.
+extract_positions_from_Rle <- function(x, i)
+{
+    if (!is.integer(i))
+        stop("'i' must be an integer vector")
+    extract_ranges_from_Rle(x, i, rep.int(1L, length(i)))
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Subsetting
+###
+
+setMethod("extractROWS", c("Rle", "ANY"),
+    function (x, i) 
+    {
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+        callGeneric()
+    }
+)
+
+setMethod("extractROWS", c("Rle", "RangeNSBS"),
     function(x, i)
     {
-        ## FIXME: Right now, the subscript 'i' is turned into an IRanges
-        ## object so we need stuff that lives in the IRanges package for this
-        ## to work. This is ugly/hacky and needs to be fixed (thru a redesign
-        ## of this method).
-        if (!requireNamespace("IRanges", quietly=TRUE))
-            stop("Couldn't load the IRanges package. You need to install ",
-                 "the IRanges\n  package in order to subset an Rle object.")
+        range <- i@subscript
+        range_start <- range[[1L]]
+        range_end <- range[[2L]]
+        ans <- extract_range_from_Rle(x, range_start, range_end)
+        mcols(ans) <- extractROWS(mcols(x), i)
+        ans
+    }
+)
 
-        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
-        ## TODO: Maybe make this the coercion method from NSBS to Ranges.
-        if (is(i, "RangesNSBS")) {
-            ir <- i@subscript
-        } else {
-            ir <- as(as.integer(i), "IRanges")
-        }
-        ## Rle_seqselect .Call entry point will segfault if 'ir' contains
-        ## empty ranges!
-        ir <- ir[width(ir) != 0L]
-        ansList <- .Call2("Rle_seqselect", x, start(ir), width(ir),
-                          PACKAGE="S4Vectors")
-        ans_values <- ansList[["values"]]
-        ans_lengths <- ansList[["lengths"]]
-        if (is.factor(runValue(x)))
-            attributes(ans_values) <- list(levels=levels(x), class="factor")
-        ans <- Rle(ans_values, ans_lengths)
-        ans <- as(ans, class(x))
-        mcols(ans) <- extractROWS(mcols(ans), i)
+setMethod("extractROWS", c("Rle", "NSBS"),
+    function(x, i)
+    {
+        ans <- extract_positions_from_Rle(x, as.integer(i))
+        mcols(ans) <- extractROWS(mcols(x), i)
         ans
     }
 )
@@ -207,6 +265,9 @@ setMethod("[", "Rle",
     }
 )
 
+### The replaced elements in 'x' must get their metadata columns from 'value'.
+### See this thread on bioc-devel:
+###   https://stat.ethz.ch/pipermail/bioc-devel/2015-November/008319.html
 setMethod("replaceROWS", "Rle",
     function(x, i, value)
     {
@@ -221,8 +282,11 @@ setMethod("replaceROWS", "Rle",
 
         i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
         lv <- length(value)
-        if (lv != 1L) 
-            return(Rle(replaceROWS(decodeRle(x), i, as.vector(value))))
+        if (lv != 1L) {
+            ans <- Rle(replaceROWS(decodeRle(x), i, as.vector(value)))
+            mcols(ans) <- replaceROWS(mcols(x), i, mcols(value))
+            return(ans)
+        }
 
         ## From here, 'value' is guaranteed to be of length 1.
 
@@ -238,9 +302,9 @@ setMethod("replaceROWS", "Rle",
 
         isFactorRle <- is.factor(runValue(x))
         value <- normalizeSingleBracketReplacementValue(value, x)
-        value <- as.vector(value)
+        value2 <- as.vector(value)
         if (isFactorRle) {
-            value <- factor(value, levels=levels(x))
+            value2 <- factor(value2, levels=levels(x))
             dummy_value <- factor(levels(x), levels=levels(x))
         }
         if (anyMissingOrOutside(start(ir), 1L, length(x)) ||
@@ -297,14 +361,15 @@ setMethod("replaceROWS", "Rle",
         if (length(valueWidths) > 0L) {
             subseqs[seq(2L, length(subseqs), by=2L)] <-
                 lapply(seq_len(length(valueWidths)), function(i)
-                       list(values=value,
+                       list(values=value2,
                             lengths=valueWidths[i]))
         }
         values <- unlist(lapply(subseqs, "[[", "values"))
         if (isFactorRle)
             values <- dummy_value[values]
-        Rle(values=values,
-            lengths=unlist(lapply(subseqs, "[[", "lengths")))
+        ans <- Rle(values, unlist(lapply(subseqs, "[[", "lengths")))
+        mcols(ans) <- replaceROWS(mcols(x), i, mcols(value))
+        ans
     }
 )
 
@@ -327,17 +392,201 @@ setReplaceMethod("[", "Rle",
     }
 )
 
-setMethod("%in%", "Rle",
-          function(x, table)
-              Rle(values = runValue(x) %in% table, lengths = runLength(x),
-                  check = FALSE))
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Subsetting an object by an Rle subscript.
+###
+### See R/subsetting-utils.R for more information.
+###
+
+setClass("RleNSBS",      # not exported
+    contains="NSBS",
+    representation(
+        subscript="Rle"  # integer-Rle
+    ),
+    prototype(
+        ## Calling Rle(integer(0)) below causes the following error at
+        ## installation time:
+        ##     Error in .Call(.NAME, ..., PACKAGE = PACKAGE) : 
+        ##       "Rle_constructor" not available for .Call() for package
+        ##       "S4Vectors"
+        ##     Error : unable to load R code in package ‘S4Vectors’
+        ##     ERROR: lazy loading failed for package ‘S4Vectors’
+        #subscript=Rle(integer(0))
+        subscript=new2("Rle", values=integer(0),
+                              lengths=integer(0),
+                              check=FALSE)
+    )
+)
+
+### Construction methods.
+### Supplied arguments are trusted so we don't check them!
+
+setMethod("NSBS", "Rle",
+    function(i, x, exact=TRUE, upperBoundIsStrict=TRUE)
+    {
+        x_NROW <- NROW(x)
+        i_vals <- runValue(i)
+        if (is.logical(i_vals) && length(i_vals) != 0L) {
+            if (anyMissing(i_vals))
+                stop("subscript contains NAs")
+            if (length(i) < x_NROW)
+                i <- rep(i, length.out=x_NROW)
+            ## The coercion method from Rle to NormalIRanges is defined in the
+            ## IRanges package.
+            if (requireNamespace("IRanges", quietly=TRUE)) {
+                i <- as(i, "NormalIRanges")
+                ## This will call the "NSBS" method for Ranges objects defined
+                ## in the IRanges package and return a RangesNSBS, or
+                ## RangeNSBS, or NativeNSBS object.
+                return(callGeneric())
+            }
+            warning(wmsg(
+                "Couldn't load the IRanges package. Installing this package ",
+                "will enable efficient subsetting by a logical-Rle object ",
+                "so is higly recommended."
+            ))
+            i <- which(i)
+            return(callGeneric())  # will return a NativeNSBS object
+        }
+        i_vals <- as.integer(NSBS(i_vals, x,
+                                  exact=exact,
+                                  upperBoundIsStrict=upperBoundIsStrict))
+        runValue(i) <- i_vals
+        new2("RleNSBS", subscript=i,
+                        upper_bound=x_NROW,
+                        upper_bound_is_strict=upperBoundIsStrict,
+                        check=FALSE)
+    }
+)
+
+### Other methods.
+
+setMethod("as.integer", "RleNSBS", function(x) decodeRle(x@subscript))
+
+setMethod("length", "RleNSBS", function(x) length(x@subscript))
+
+setMethod("anyDuplicated", "RleNSBS",
+    function(x, incomparables=FALSE, ...) anyDuplicated(x@subscript)
+)
+
+setMethod("isStrictlySorted", "RleNSBS",
+    function(x) isStrictlySorted(x@subscript)
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Subsetting an Rle object by an Rle subscript.
+###
+
+### Simplified version of rep.int() for Rle objects. Handles only the case
+### where 'times' has the length of 'x'.
+.rep_times_Rle <- function(x, times)
+{
+    breakpoints <- end(x)
+    if (length(times) != last_or(breakpoints, 0L))
+        stop("invalid 'times' argument")
+    runLength(x) <- groupsum(times, breakpoints)
+    x
+}
+
+setMethod("extractROWS", c("Rle", "RleNSBS"),
+    function(x, i)
+    {
+        rle <- i@subscript
+        .rep_times_Rle(extractROWS(x, runValue(rle)), runLength(rle))
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Other subsetting-related operations
+###
+
+### S3/S4 combo for rev.Rle
+rev.Rle <- function(x)
+{
+    x@values <- rev(runValue(x))
+    x@lengths <- rev(runLength(x))
+    x
+}
+setMethod("rev", "Rle", rev.Rle)
+
+setMethod("rep.int", "Rle",
+    function(x, times)
+    {
+        if (!is.numeric(times))
+            stop("invalid 'times' argument")
+        if (!is.integer(times))
+            times <- as.integer(times)
+        if (anyMissingOrOutside(times, 0L))
+            stop("invalid 'times' argument")
+
+        x_len <- length(x)
+        times_len <- length(times)
+        if (times_len == x_len)
+            return(.rep_times_Rle(x, times))
+        if (times_len != 1L)
+            stop("invalid 'times' argument")
+        ans <- Rle(rep.int(runValue(x), times),
+                   rep.int(runLength(x), times))
+        as(ans, class(x))  # rep.int() must act like an endomorphism
+    }
+)
+
+setMethod("rep", "Rle",
+          function(x, times, length.out, each)
+          {
+              usedEach <- FALSE
+              if (!missing(each) && length(each) > 0) {
+                  each <- as.integer(each[1L])
+                  if (!is.na(each)) {
+                      if (each < 0)
+                          stop("invalid 'each' argument")
+                      usedEach <- TRUE
+                      if (each == 0)
+                          x <- new2(class(x), values=runValue(x)[0L],
+                                              check=FALSE)
+                      else
+                          x@lengths <- each[1L] * runLength(x)
+                  }
+              }
+              if (!missing(length.out) && length(length.out) > 0) {
+                  n <- length(x)
+                  length.out <- as.integer(length.out[1L])
+                  if (!is.na(length.out)) {
+                      if (length.out == 0) {
+                          x <- new2(class(x), values=runValue(x)[0L],
+                                              check=FALSE)
+                      } else if (length.out < n) {
+                          x <- window(x, 1, length.out)
+                      } else if (length.out > n) {
+                          if (n == 0) {
+                              x <- Rle(rep(runValue(x), length.out=1),
+                                       length.out)
+                          } else {
+                              x <-
+                                window(rep.int(x, ceiling(length.out / n)),
+                                       1, length.out)
+                          }
+                      }
+                  }
+              } else if (!missing(times)) {
+                  if (usedEach && length(times) != 1)
+                      stop("invalid 'times' argument")
+                  x <- rep.int(x, times)
+              }
+              x
+          })
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Combining.
+###
 
 setMethod("c", "Rle", 
           function(x, ..., recursive = FALSE)
           {
-              if (!identical(recursive, FALSE))
-                  stop("\"c\" method for Rle objects ",
-                       "does not support the 'recursive' argument")
               args <- lapply(unname(list(x, ...)), Rle)
               args <- args[sapply(args, length) > 0]
               if (length(args) == 0L)
@@ -347,6 +596,24 @@ setMethod("c", "Rle",
               Rle(ans_values, ans_lengths)
           })
 
+setMethod("append", c("Rle", "vector"),
+          function (x, values, after = length(x)) {
+              append(x, Rle(values), after)
+          })
+
+setMethod("append", c("vector", "Rle"),
+          function (x, values, after = length(x)) {
+              append(Rle(x), values, after)
+          })
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Other methods.
+###
+
+setMethod("%in%", "Rle",
+          function(x, table)
+              new_Rle(runValue(x) %in% table, runLength(x), check=FALSE))
+
 setGeneric("findRun", signature = "vec",
            function(x, vec) standardGeneric("findRun"))
 
@@ -360,8 +627,7 @@ setMethod("findRun", signature = c(vec = "Rle"),
 
 setMethod("is.na", "Rle",
           function(x)
-              Rle(values = is.na(runValue(x)), lengths = runLength(x),
-                  check = FALSE))
+              new_Rle(is.na(runValue(x)), runLength(x), check=FALSE))
 
 setMethod("anyNA", "Rle",
           function(x)
@@ -376,15 +642,17 @@ setMethod("is.unsorted", "Rle",
               ans
           })
 
-setMethod("length", "Rle", function(x) sum(runLength(x)))
-
 setMethod("match", c("ANY", "Rle"),
     function(x, table, nomatch=NA_integer_, incomparables=NULL)
     {
-        table_run_starts <- start(table)
-        table <- runValue(table)
-        m <- callGeneric()
-        table_run_starts[m]
+        m <- match(x, runValue(table), incomparables=incomparables)
+        ans <- start(table)[m]
+        ## 'as.integer(nomatch)[1L]' seems to mimic how base::match() treats
+        ## the 'nomatch' argument.
+        nomatch <- as.integer(nomatch)[1L]
+        if (!is.na(nomatch))
+            ans[is.na(ans)] <- nomatch
+        ans
     }
 )
 
@@ -408,187 +676,27 @@ setMethod("match", c("Rle", "Rle"),
     }
 )
 
-setMethod("rep", "Rle",
-          function(x, times, length.out, each)
-          {
-              usedEach <- FALSE
-              if (!missing(each) && length(each) > 0) {
-                  each <- as.integer(each[1L])
-                  if (!is.na(each)) {
-                      if (each < 0)
-                          stop("invalid 'each' argument")
-                      usedEach <- TRUE
-                      if (each == 0)
-                          x <- new(class(x), values = runValue(x)[0L])
-                      else
-                          x@lengths <- each[1L] * runLength(x)
-                  }
-              }
-              if (!missing(length.out) && length(length.out) > 0) {
-                  n <- length(x)
-                  length.out <- as.integer(length.out[1L])
-                  if (!is.na(length.out)) {
-                      if (length.out == 0) {
-                          x <- new(class(x), values = runValue(x)[0L])
-                      } else if (length.out < n) {
-                          x <- window(x, 1, length.out)
-                      } else if (length.out > n) {
-                          if (n == 0) {
-                              x <- Rle(rep(runValue(x), length.out=1),
-                                       length.out)
-                          } else {
-                              x <-
-                                window(rep.int(x, ceiling(length.out / n)),
-                                       1, length.out)
-                          }
-                      }
-                  }
-              } else if (!missing(times)) {
-                  if (usedEach && length(times) != 1)
-                      stop("invalid 'times' argument")
-                  x <- rep.int(x, times)
-              }
-              x
-          })
-
-setMethod("rep.int", "Rle",
-          function(x, times)
-          {
-              n <- length(x)
-              if (!is.integer(times))
-                  times <- as.integer(times)
-              if ((length(times) > 1 && length(times) < n) ||
-                  anyMissingOrOutside(times, 0L))
-                  stop("invalid 'times' argument")
-              if (length(times) == n) {
-                  runLength(x) <- diffWithInitialZero(cumsum(times)[end(x)])
-              } else if (length(times) == 1) {
-                  times <- as.vector(times)
-                  x <- Rle(values  = rep.int(runValue(x), times),
-                           lengths = rep.int(runLength(x), times))
-              }
-              x
-          })
-
-### S3/S4 combo for rev.Rle
-rev.Rle <- function(x)
-{
-    x@values <- rev(runValue(x))
-    x@lengths <- rev(runLength(x))
-    x
-}
-setMethod("rev", "Rle", rev.Rle)
-
-setGeneric("shiftApply", signature = c("X", "Y"),
-           function(SHIFT, X, Y, FUN, ..., OFFSET = 0L, simplify = TRUE,
-                    verbose = FALSE)
-           standardGeneric("shiftApply"))
-
-setMethod("shiftApply", signature(X = "Rle", Y = "Rle"),
-          function(SHIFT, X, Y, FUN, ..., OFFSET = 0L, simplify = TRUE,
-                   verbose = FALSE)
-          {
-              FUN <- match.fun(FUN)
-              N <- length(X)
-              if (N != length(Y))
-                  stop("'X' and 'Y' must be of equal length")
-
-              if (!is.integer(SHIFT))
-                  SHIFT <- as.integer(SHIFT)
-              if (length(SHIFT) == 0 || anyMissingOrOutside(SHIFT, 0L))
-                  stop("all 'SHIFT' values must be non-negative")
-
-              if (!is.integer(OFFSET))
-                  OFFSET <- as.integer(OFFSET)
-              if (length(OFFSET) == 0 || anyMissingOrOutside(OFFSET, 0L))
-                  stop("'OFFSET' must be non-negative")
-
-              ## Perform X setup
-              infoX <-
-                getStartEndRunAndOffset(X, rep.int(1L + OFFSET, length(SHIFT)),
-                                        N - SHIFT)
-              runStartX <- infoX[["start"]][["run"]]
-              offsetStartX <- infoX[["start"]][["offset"]]
-              runEndX <- infoX[["end"]][["run"]]
-              offsetEndX <- infoX[["end"]][["offset"]]
-
-              ## Perform Y setup
-              infoY <-
-                getStartEndRunAndOffset(Y, 1L + SHIFT,
-                                        rep.int(N - OFFSET, length(SHIFT)))
-              runStartY <- infoY[["start"]][["run"]]
-              offsetStartY <- infoY[["start"]][["offset"]]
-              runEndY <- infoY[["end"]][["run"]]
-              offsetEndY <- infoY[["end"]][["offset"]]
-
-              ## Performance Optimization
-              ## Use a stripped down loop with empty Rle object
-              newX <- new("Rle")
-              newY <- new("Rle")
-              if (verbose) {
-                  maxI <- length(SHIFT)
-                  ans <-
-                    sapply(seq_len(length(SHIFT)),
-                           function(i) {
-                               cat("\r", i, "/", maxI)
-                               FUN(.Call2("Rle_window",
-                                         X, runStartX[i], runEndX[i],
-                                         offsetStartX[i], offsetEndX[i],
-                                         newX, PACKAGE = "S4Vectors"),
-                                   .Call2("Rle_window",
-                                         Y, runStartY[i], runEndY[i],
-                                         offsetStartY[i], offsetEndY[i],
-                                         newY, PACKAGE = "S4Vectors"),
-                                              ...)
-                           }, simplify = simplify)
-                  cat("\n")
-              } else {
-                  ans <-
-                    sapply(seq_len(length(SHIFT)),
-                           function(i)
-                               FUN(.Call2("Rle_window",
-                                         X, runStartX[i], runEndX[i],
-                                         offsetStartX[i], offsetEndX[i],
-                                         newX, PACKAGE = "S4Vectors"),
-                                   .Call2("Rle_window",
-                                         Y, runStartY[i], runEndY[i],
-                                         offsetStartY[i], offsetEndY[i],
-                                         newY, PACKAGE = "S4Vectors"),
-                                   ...),
-                               simplify = simplify)
-              }
-              ans
-          })
-
 ### FIXME: Remove in R 3.3
 setMethod("order", "Rle",
-    function(..., na.last=TRUE, decreasing=FALSE)
+    function(..., na.last=TRUE, decreasing=FALSE, method=c("shell", "radix"))
     {
         args <- lapply(unname(list(...)), decodeRle)
-        do.call(order, c(args, list(na.last=na.last, decreasing=decreasing)))
+        do.call(order, c(args, list(na.last=na.last,
+                                    decreasing=decreasing,
+                                    method=method)))
     }
 )
 
-### S3/S4 combo for sort.Rle
 .sort.Rle <- function(x, decreasing=FALSE, na.last=NA, ...)
 {
     if (is.na(na.last)) {
         if (anyMissing(runValue(x)))
             x <- x[!is.na(x)]
     }
-    if (is.integer(runValue(x)) || is.factor(runValue(x)))
-        ord <- orderInteger(runValue(x), decreasing=decreasing,
-                                         na.last=na.last)
-    else
-        ord <- order(runValue(x), decreasing=decreasing,
-                     na.last=na.last)
-    Rle(values=runValue(x)[ord], 
-        lengths=runLength(x)[ord],
-        check=FALSE)
+    ord <- base::order(runValue(x), na.last=na.last, decreasing=decreasing)
+    new_Rle(runValue(x)[ord], runLength(x)[ord], check=FALSE)
 }
-sort.Rle <- function(x, decreasing=FALSE, ...)
-    .sort.Rle(x, decreasing=decreasing, ...)
-setMethod("sort", "Rle", sort.Rle)
+setMethod("sort", "Rle", .sort.Rle)
 
 setMethod("xtfrm", "Rle", function(x) {
     initialize(x, values=xtfrm(runValue(x)))
@@ -658,17 +766,13 @@ setMethod("table", "Rle",
     }
 }
 
-### S3/S4 combo for duplicated.Rle
 .duplicated.Rle <- function(x, incomparables=FALSE, fromLast=FALSE)
     stop("no \"duplicated\" method for Rle objects yet, sorry")
-duplicated.Rle <- function(x, incomparables=FALSE, ...)
-    .duplicated.Rle(x, incomparables=incomparables, ...)
-setMethod("duplicated", "Rle", duplicated.Rle)
+setMethod("duplicated", "Rle", .duplicated.Rle)
 
-### S3/S4 combo for unique.Rle
-unique.Rle <- function(x, incomparables=FALSE, ...)
+.unique.Rle <- function(x, incomparables=FALSE, ...)
     unique(runValue(x), incomparables=incomparables, ...)
-setMethod("unique", "Rle", unique.Rle)
+setMethod("unique", "Rle", .unique.Rle)
 
 ### S3/S4 combo for anyDuplicated.Rle
 anyDuplicated.Rle <- function(x, incomparables=FALSE, ...)
@@ -679,12 +783,14 @@ setMethod("isStrictlySorted", "Rle",
     function(x)  all(runLength(x) == 1L) && isStrictlySorted(runValue(x))
 )
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Set methods
 ###
 ### The return values of these do not have any duplicated values, so
 ### it would obviously be more efficient to return plain vectors. That
 ### might violate user expectations though.
+###
 
 setMethod("union", c("Rle", "Rle"), function(x, y) {
   Rle(union(runValue(x), runValue(y)))
@@ -722,6 +828,7 @@ setMethod("setdiff", c("Rle", "ANY"), function(x, y) {
   Rle(setdiff(runValue(x), as.vector(y)))
 })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The "show" method
 ###
@@ -758,3 +865,4 @@ setMethod("show", "Rle",
           })
 
 setMethod("showAsCell", "Rle", function(object) as.vector(object))
+
