@@ -56,9 +56,9 @@ setMethod("Ops", signature(e1 = "Rle", e2 = "Rle"),
                   which1 <- findIntervalAndStartFromWidth(ends, runLength(e1))[["interval"]]
                   which2 <- findIntervalAndStartFromWidth(ends, runLength(e2))[["interval"]]
               }
-              Rle(values = callGeneric(runValue(e1)[which1], runValue(e2)[which2]),
-                  lengths = diffWithInitialZero(ends),
-                  check = FALSE)
+              new_Rle(callGeneric(runValue(e1)[which1], runValue(e2)[which2]),
+                      diffWithInitialZero(ends),
+                      check=FALSE)
           })
 
 setMethod("Ops", signature(e1 = "Rle", e2 = "vector"),
@@ -80,7 +80,7 @@ setMethod("Math", "Rle",
                          values <- cumsum(as.vector(y))
                          lengths <- rep.int(1L, length(values))
                          lengths[startZero - c(0L, cumsum(head(widthZero, -1) - 1L))] <- widthZero
-                         Rle(values = values, lengths = lengths, check = FALSE)
+                         new_Rle(values, lengths, check=FALSE)
                      },
                      cumprod =
                      {
@@ -92,18 +92,18 @@ setMethod("Math", "Rle",
                          values <- cumprod(as.vector(y))
                          lengths <- rep.int(1L, length(values))
                          lengths[startOne - c(0L, cumsum(head(widthOne, -1) - 1L))] <- widthOne
-                         Rle(values = values, lengths = lengths, check = FALSE)
+                         new_Rle(values, lengths, check=FALSE)
                      },
-                     Rle(values = callGeneric(runValue(x)),
-                         lengths = runLength(x), check = FALSE)))
+                     new_Rle(callGeneric(runValue(x)),
+                             runLength(x), check=FALSE)))
 
 setMethod("Math2", "Rle",
           function(x, digits)
           {
               if (missing(digits))
                   digits <- ifelse(.Generic == "round", 0, 6)
-              Rle(values = callGeneric(runValue(x), digits = digits),
-                  lengths = runLength(x), check = FALSE)
+              new_Rle(callGeneric(runValue(x), digits = digits),
+                      runLength(x), check=FALSE)
           })
 
 setMethod("Summary", "Rle",
@@ -134,8 +134,7 @@ setMethod("Summary", "Rle",
 
 setMethod("Complex", "Rle",
           function(z)
-              Rle(values = callGeneric(runValue(z)), lengths = runLength(z),
-                  check = FALSE))
+              new_Rle(callGeneric(runValue(z)), runLength(z), check=FALSE))
 
 ### S3/S4 combo for summary.Rle
 summary.Rle <- function(object, ..., digits=max(3, getOption("digits") - 3)) 
@@ -175,7 +174,7 @@ setMethod("summary", "Rle", summary.Rle)
 
 setMethod("!", "Rle",
           function(x)
-              Rle(values = !runValue(x), lengths = runLength(x), check = FALSE))
+              new_Rle(!runValue(x), runLength(x), check=FALSE))
 
 setMethod("which", "Rle",
           function(x, arr.ind = FALSE) {
@@ -196,36 +195,23 @@ setMethod("which.max", "Rle",
           })
 
 ## base::ifelse works fine for S4 'test', but not for S4 yes/no
-.ifelse_generic_deprecation_msg <- c(
-    "  The \"ifelse\" methods for Rle objects are deprecated. Please use",
+.ifelse_generic_defunct_msg <- c(
+    "  The \"ifelse\" methods for Rle objects are defunct. Please use",
     "\n\n      as(ifelse(test, as.vector(yes), as.vector(no)), \"Rle\")",
     "\n\n  instead."
 )
 setMethod("ifelse", c(yes = "Rle"), function(test, yes, no) 
-          {
-            .Deprecated(msg=.ifelse_generic_deprecation_msg)
-            yes <- as.vector(yes)
-            as(callGeneric(), "Rle")
-          })
+            .Defunct(msg=.ifelse_generic_defunct_msg))
 setMethod("ifelse", c(no = "Rle"), function(test, yes, no) 
-          {
-            .Deprecated(msg=.ifelse_generic_deprecation_msg)
-            no <- as.vector(no)
-            as(callGeneric(), "Rle")
-          })
+            .Defunct(msg=.ifelse_generic_defunct_msg))
 setMethod("ifelse", c(yes = "Rle", no = "Rle"), function(test, yes, no) 
-          {
-            .Deprecated(msg=.ifelse_generic_deprecation_msg)
-            yes <- as.vector(yes)
-            no <- as.vector(no)
-            as(callGeneric(), "Rle")
-          })
+            .Defunct(msg=.ifelse_generic_defunct_msg))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Other numerical data methods
 ###
 
-### S3/S4 combo for diff.Rle
+diff.Rle <- function(x, ...) diff(x, ...)
 .diff.Rle <- function(x, lag = 1, differences = 1)
 {
     if (!isSingleNumber(lag) || lag < 1L ||
@@ -241,7 +227,6 @@ setMethod("ifelse", c(yes = "Rle", no = "Rle"), function(test, yes, no)
     }
     x
 }
-diff.Rle <- function(x, ...) .diff.Rle(x, ...)
 setMethod("diff", "Rle", .diff.Rle)
 
 .psummary.Rle <- function(FUN, ..., MoreArgs = NULL) {
@@ -251,8 +236,7 @@ setMethod("diff", "Rle", .diff.Rle)
         for (i in 2:length(args))
             ends <- sortedMerge(ends, end(args[[i]]))
     }
-    Rle(values =
-        do.call(FUN,
+    new_Rle(do.call(FUN,
                 c(lapply(args,
                          function(x) {
                              runs <- findIntervalAndStartFromWidth(ends,
@@ -260,7 +244,7 @@ setMethod("diff", "Rle", .diff.Rle)
                              runValue(x)[runs]
                          }),
                  MoreArgs)),
-        lengths = diffWithInitialZero(ends), check = FALSE)
+            diffWithInitialZero(ends), check=FALSE)
 }
 
 setMethod("pmax", "Rle", function(..., na.rm = FALSE)
@@ -539,9 +523,10 @@ setMethod("runq", "Rle",
 ###
 
 setMethod("nchar", "Rle",
-          function(x, type = "chars", allowNA = FALSE)
-              Rle(values = nchar(runValue(x), type = type, allowNA = allowNA),
-                  lengths = runLength(x), check = FALSE))
+    function(x, type="chars", allowNA=FALSE, keepNA=NA)
+        new_Rle(nchar(runValue(x), type=type, allowNA=allowNA, keepNA=keepNA),
+                runLength(x), check=FALSE)
+)
 
 setMethod("substr", "Rle",
           function(x, start, stop)
@@ -668,8 +653,7 @@ setMethod("gsub", signature = c(pattern = "ANY", replacement = "ANY", x = "Rle")
           paste(runValue(e1)[which1], runValue(e2)[which2], sep = sep,
                 collapse = collapse)
     }
-    Rle(values = values, lengths = diffWithInitialZero(ends),
-        check = FALSE)
+    new_Rle(values, diffWithInitialZero(ends), check=FALSE)
 }
 
 setMethod("paste", "Rle",
@@ -718,20 +702,18 @@ setMethod("levels", "Rle", levels.Rle)
 setReplaceMethod("levels", "Rle",
                  function(x, value) {
                      levels(x@values) <- value
-                     if (anyDuplicated(value)) {
-                         x <-
-                           Rle(values = runValue(x), lengths = runLength(x),
-                               check = FALSE)
-                     }
+                     if (anyDuplicated(value))
+                         x <- new_Rle(runValue(x), runLength(x), check=FALSE)
                      x
                  })
 
-droplevels.Rle <- function(x) {
+droplevels.Rle <- function(x, ...) droplevels(x, ...)
+.droplevels.Rle <- function(x) {
   if (!is.factor(runValue(x))) {
     stop("levels can only be dropped when runValue(x) is a factor")
   }
   runValue(x) <- droplevels(runValue(x))
   x
 }
-setMethod("droplevels", "Rle", droplevels.Rle)
+setMethod("droplevels", "Rle", .droplevels.Rle)
 
