@@ -70,7 +70,7 @@ SimpleList <- function(...)
     ##   "list"
     if (length(args) == 1L && extends(class(args[[1L]]), "list"))
         args <- args[[1L]]
-    new("SimpleList", listData=args)
+    new2("SimpleList", listData=args, check=FALSE)
 }
 
 
@@ -111,16 +111,27 @@ setMethod("extractROWS", "SimpleList",
     function(x, i)
     {
         i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
-        initialize(x, listData=extractROWS(x@listData, i),
-                      elementMetadata=extractROWS(x@elementMetadata, i))
+        ans_listData <- extractROWS(x@listData, i)
+        ans_elementMetadata <- extractROWS(x@elementMetadata, i)
+        BiocGenerics:::replaceSlots(x,
+            listData=ans_listData,
+            elementMetadata=ans_elementMetadata)
     }
 )
 
+### The replaced elements in 'x' must get their metadata columns from 'value'.
+### See this thread on bioc-devel:
+###   https://stat.ethz.ch/pipermail/bioc-devel/2015-November/008319.html
 setMethod("replaceROWS", "SimpleList",
     function(x, i, value)
     {
         i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
-        initialize(x, listData=replaceROWS(x@listData, i, value@listData))
+        ans_listData <- replaceROWS(x@listData, i, value@listData)
+        ans_elementMetadata <- replaceROWS(x@elementMetadata, i,
+                                           value@elementMetadata)
+        BiocGenerics:::replaceSlots(x,
+            listData=ans_listData,
+            elementMetadata=ans_elementMetadata)
     }
 )
 
@@ -203,9 +214,7 @@ setMethod("mendoapply", "SimpleList",
         names(ans) <- NULL
     ans
 }
-### S3/S4 combo for as.list.SimpleList
-as.list.SimpleList <- function(x, ...) .as.list.SimpleList(x, ...)
-setMethod("as.list", "SimpleList", as.list.SimpleList)
+setMethod("as.list", "SimpleList", .as.list.SimpleList)
 
 setAs("ANY", "SimpleList", function(from) {
   coerceToSimpleList(from)
