@@ -48,7 +48,8 @@ setMethod("parallelSlotNames", "Vector", function(x) "elementMetadata")
 ### parallelVectorNames() is for internal use only.
 setGeneric("parallelVectorNames",
            function(x) standardGeneric("parallelVectorNames"))
-setMethod("parallelVectorNames", "ANY", function(x) character())
+setMethod("parallelVectorNames", "ANY",
+          function(x) setdiff(colnames(as.data.frame(new(class(x)))), "value"))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -64,7 +65,7 @@ setMethod("lengths", "Vector",
      {
          if (!isTRUEorFALSE(use.names))
              stop("'use.names' must be TRUE or FALSE")
-         ans <- elementLengths(x)
+         ans <- elementNROWS(x)  # This is wrong! See ?Vector for the details.
          if (!use.names)
              names(ans) <- NULL
          ans
@@ -142,7 +143,12 @@ setMethod("anyNA", "Vector", function(x) any(is.na(x)))
     for (slotname in x_pslotnames) {
         tmp <- slot(x, slotname)
         if (!(is.null(tmp) || NROW(tmp) == x_len)) {
-            msg <- c("'x@", slotname, "' is not parallel to 'x'")
+            if (slotname == "elementMetadata") {
+                what <- "mcols(x)"
+            } else {
+                what <- paste0("x@", slotname)
+            }
+            msg <- c("'", what, "' is not parallel to 'x'")
             return(paste(msg, collapse=""))
         }
     }
@@ -238,12 +244,23 @@ setAs("Vector", "factor", function(from) as.factor(from))
 setAs("Vector", "data.frame", function(from) as.data.frame(from))
 
 ### S3/S4 combo for as.data.frame.Vector
-as.data.frame.Vector <- function(x, row.names=NULL, optional=FALSE, ...)
-{
-    x <- as.vector(x)
+as.data.frame.Vector <- function(x, row.names=NULL, optional=FALSE, ...) {
     as.data.frame(x, row.names=NULL, optional=optional, ...)
 }
-setMethod("as.data.frame", "Vector", as.data.frame.Vector)
+setMethod("as.data.frame", "Vector",
+          function(x, row.names=NULL, optional=FALSE, ...)
+          {
+              x <- as.vector(x)
+              as.data.frame(x, row.names=row.names, optional=optional, ...)
+          })
+
+as.matrix.Vector <- function(x, ...) {
+    as.matrix(x)
+}
+
+setMethod("as.matrix", "Vector", function(x) {
+              as.matrix(as.vector(x))
+          })
 
 makeFixedColumnEnv <- function(x, parent, tform = identity) {
   env <- new.env(parent=parent)
@@ -264,6 +281,8 @@ setMethod("as.env", "Vector", function(x, enclos, tform = identity) {
   addSelfRef(x, makeFixedColumnEnv(x, as.env(mcols(x), enclos, tform), tform))
 })
 
+as.list.Vector <- function(x, ...) as.list(x, ...)
+setMethod("as.list", "Vector", function(x, ...) as.list(as(x, "List"), ...))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Setters.
@@ -272,20 +291,39 @@ setMethod("as.env", "Vector", function(x, enclos, tform = identity) {
 setGeneric("elementMetadata<-",
            function(x, ..., value) standardGeneric("elementMetadata<-"))
 
+### NOT exported but used in packages IRanges, GenomicRanges,
+### SummarizedExperiment, GenomicAlignments, and maybe more...
+### 3x faster than new("DataFrame", nrows=nrow).
+### 500x faster than DataFrame(matrix(nrow=nrow, ncol=0L)).
+make_zero_col_DataFrame <- function(nrow)
+    new2("DataFrame", nrows=nrow, check=FALSE)
+
+.normalize_mcols_replacement_value <- function(value, x)
+{
+    x_slots <- getSlots(class(x))
+    ## Should never happen because 'x' should always be a Vector object so
+    ## should always have the 'elementMetadata' slot.
+    if (!("elementMetadata" %in% names(x_slots)))
+        stop(wmsg("trying to set metadata columns on an object that does ",
+                  "not support them (i.e. with no 'elementMetadata' slot)"))
+    mcols_class <- x_slots[["elementMetadata"]]
+    if (is.null(value)) {
+        if (is(NULL, mcols_class))
+            return(NULL)
+        value <- make_zero_col_DataFrame(length(x))
+    }
+    value <- as(value, mcols_class, strict=TRUE)
+    ## From here 'value' is guaranteed to be a DataTable object.
+    if (!is.null(rownames(value)))
+        rownames(value) <- NULL
+    V_recycle(value, x, x_what="value", skeleton_what="x")
+}
+
 setReplaceMethod("elementMetadata", "Vector",
     function(x, ..., value)
     {
-        if (!is(value, "DataTableORNULL"))
-            stop("replacement 'elementMetadata' value must be ",
-                 "a DataTable object or NULL")
-        if ("elementMetadata" %in% names(attributes(x))) {
-            if (!is.null(value) && length(x) != nrow(value))
-                stop("supplied metadata columns must have the length of 'x'")
-            if (!is.null(value))
-                rownames(value) <- NULL
-            x@elementMetadata <- value
-        }
-        x
+        value <- .normalize_mcols_replacement_value(value, x)
+        BiocGenerics:::replaceSlots(x, elementMetadata=value, check=FALSE)
     }
 )
 
@@ -321,7 +359,6 @@ setGeneric("rename", function(x, ...) standardGeneric("rename"))
 setMethod("rename", "vector", .renameVector)
 setMethod("rename", "Vector", .renameVector)
 
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
 ###
@@ -394,15 +431,21 @@ setMethod("replaceROWS", "Vector",
     {
         idx <- seq_along(x)
         i <- extractROWS(setNames(idx, names(x)), i)
-        ## Assuming that objects of class 'class(x)' can be combined with c().
+        ## Assuming that c() works on objects of class 'class(x)' and does the
+        ## right thing (i.e. returns an object of the same class as 'x' and of
+        ## length 'length(x) + length(value)').
         ans <- c(x, value)
-        idx[i] <- length(x) + seq_len(length(value))
         ## Assuming that [ works on objects of class 'class(x)'.
+        idx[i] <- length(x) + seq_along(value)
         ans <- ans[idx]
         ## Restore the original decoration.
         metadata(ans) <- metadata(x)
         names(ans) <- names(x)
-        mcols(ans) <- mcols(x)
+        ## However, we want the replaced elements in 'x' to get their
+        ## metadata columns from 'value' so we do not restore the original
+        ## metadata columns. See this thread on bioc-devel:
+        ##  https://stat.ethz.ch/pipermail/bioc-devel/2015-November/008319.html
+        #mcols(ans) <- mcols(x)
         ans
     }
 )
@@ -413,16 +456,17 @@ setMethod("replaceROWS", "Vector",
 ###
 
 ### S3/S4 combo for window.Vector
-.window.Vector <- function(x, start=NA, end=NA, width=NA)
+window.Vector <- function(x, ...) window(x, ...)
+Vector_window <- function(x, start=NA, end=NA, width=NA)
 {
-    i <- WindowNSBS(x, start=start, end=end, width=width)
+    i <- RangeNSBS(x, start=start, end=end, width=width)
     extractROWS(x, i)
 }
-window.Vector <- function(x, ...) .window.Vector(x, ...)
-setMethod("window", "Vector", window.Vector)
+setMethod("window", "Vector", Vector_window)
 
 ### S3/S4 combo for head.Vector
-head.Vector <- function(x, n=6L, ...)
+head.Vector <- function(x, n=6L, ...) head(x, n, ...)
+Vector_head <- function(x, n=6L)
 {
     if (!isSingleNumber(n))
         stop("'n' must be a single integer")
@@ -436,10 +480,11 @@ head.Vector <- function(x, n=6L, ...)
     }
     window(x, start=1L, width=n)
 }
-setMethod("head", "Vector", head.Vector)
+setMethod("head", "Vector", Vector_head)
 
 ## S3/S4 combo for tail.Vector
-tail.Vector <- function(x, n=6L, ...)
+tail.Vector <- function(x, n=6L, ...) tail(x, n, ...)
+Vector_tail <- function(x, n=6L)
 {
     if (!isSingleNumber(n))
         stop("'n' must be a single integer")
@@ -453,7 +498,10 @@ tail.Vector <- function(x, n=6L, ...)
     }
     window(x, end=x_NROW, width=n)
 }
-setMethod("tail", "Vector", tail.Vector)
+setMethod("tail", "Vector", Vector_tail)
+
+## NOT exported.
+revROWS <- function(x) extractROWS(x, rev(seq_len(NROW(x))))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -509,17 +557,16 @@ setMethod("showAsCell", "Vector", function(object)
 setMethod("showAsCell", "Date", function(object) object)
 setMethod("showAsCell", "POSIXt", function(object) object)
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Combining.
 ###
 
-makeZeroColDataFrame <- function(x) new("DataFrame", nrows=length(x))
-
 ### Somewhat painful that we do not always have a DataFrame in elementMetadata
 ensureMcols <- function(x) {
   mc <- mcols(x)
   if (is.null(mc)) {
-    mc <- makeZeroColDataFrame(x)
+    mc <- make_zero_col_DataFrame(length(x))
   }
   mc
 }
@@ -533,8 +580,10 @@ rbind_mcols <- function(x, ...)
     mcols_is_null <- sapply(mcols_list, is.null)
     if (all(mcols_is_null))
         return(NULL)    
-    mcols_list[mcols_is_null] <- lapply(args[mcols_is_null],
-                                        makeZeroColDataFrame)
+    mcols_list[mcols_is_null] <- lapply(
+        args[mcols_is_null],
+        function(arg) make_zero_col_DataFrame(length(arg))
+    )
     colnames_list <- lapply(mcols_list, colnames)
     allCols <- unique(unlist(colnames_list, use.names=FALSE))
     fillCols <- function(df) {
@@ -569,6 +618,7 @@ setMethod("append", c("Vector", "Vector"),
     }
 )
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Evaluating.
 ###
@@ -588,3 +638,40 @@ setMethod("with", "Vector",
           {
             safeEval(substitute(expr), data, parent.frame(), ...)
           })
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Utilities.
+###
+
+setGeneric("expand.grid", signature="...")
+
+setMethod("expand.grid", "Vector",
+          function(..., KEEP.OUT.ATTRS = TRUE, stringsAsFactors = TRUE) {
+              args <- list(...)
+              inds <- lapply(args, seq_along)
+              grid <- do.call(expand.grid,
+                              c(inds,
+                                KEEP.OUT.ATTRS=KEEP.OUT.ATTRS,
+                                stringsAsFactors=stringsAsFactors))
+              names(args) <- names(grid)
+              ans <- DataFrame(mapply(`[`, args, grid, SIMPLIFY=FALSE),
+                               check.names=FALSE)
+              metadata(ans)$out.attrs <- attr(grid, "out.attrs")
+              ans
+          })
+
+### FIXME: tapply method still in IRanges
+setMethod("by", "Vector",
+          function(data, INDICES, FUN, ..., simplify = TRUE)
+          {
+              if (!is.list(INDICES)) {
+                  INDICES <- setNames(list(INDICES),
+                                      deparse(substitute(INDICES))[1L])
+              }
+              FUNx <- function(i) FUN(extractROWS(data, i), ...)
+              structure(tapply(seq_len(NROW(data)), INDICES, FUNx,
+                               simplify = simplify),
+                        call = match.call(), class = "by")
+          })
+
+diff.Vector <- function(x, ...) diff(x, ...)
