@@ -6,7 +6,7 @@
 
 ### Functions/operators for comparing, ordering, tabulating:
 ###
-###     compare
+###     pcompare
 ###     ==
 ###     !=
 ###     <=
@@ -38,16 +38,16 @@
 ### Element-wise (aka "parallel") comparison of 2 Vector objects.
 ###
 
-setGeneric("compare", function(x, y) standardGeneric("compare"))
+setGeneric("pcompare", function(x, y) standardGeneric("pcompare"))
 
-### The methods below are implemented on top of compare().
+### The methods below are implemented on top of pcompare().
 
 setMethods("==", .OP2_SIGNATURES,
-    function(e1, e2) { compare(e1, e2) == 0L }
+    function(e1, e2) { pcompare(e1, e2) == 0L }
 )
 
 setMethods("<=", .OP2_SIGNATURES,
-    function(e1, e2) { compare(e1, e2) <= 0L }
+    function(e1, e2) { pcompare(e1, e2) <= 0L }
 )
 
 ### The methods below are implemented on top of == and <=.
@@ -60,6 +60,8 @@ setMethods("<", .OP2_SIGNATURES, function(e1, e2) { !(e2 <= e1) })
 
 setMethods(">", .OP2_SIGNATURES, function(e1, e2) { !(e1 <= e2) })
 
+compare <- function(...) {.Deprecated("pcompare"); pcompare(...)}
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### selfmatch()
@@ -93,7 +95,7 @@ reverseSelfmatchMapping <- function(selfmatch_mapping)
 {
     ans <- vector(mode="list", length=length(selfmatch_mapping))
     sparse_ans <- split(seq_along(selfmatch_mapping), selfmatch_mapping)
-    ans[as.integer(names(sparse_ans))] <- sparse_ans
+    ans[as.integer(names(sparse_ans))] <- as.list(sparse_ans)
     ans
 }
 
@@ -107,6 +109,9 @@ reverseSelfmatchMapping <- function(selfmatch_mapping)
 
 ### S3/S4 combo for duplicated.Vector
 duplicated.Vector <- function(x, incomparables=FALSE, ...)
+    duplicated(x, incomparables=incomparables, ...)
+
+.duplicated.Vector <- function(x, incomparables=FALSE, ...)
 {
     if (!identical(incomparables, FALSE)) 
         stop("the \"duplicated\" method for Vector objects ", 
@@ -128,15 +133,17 @@ duplicated.Vector <- function(x, incomparables=FALSE, ...)
         ans <- rev(ans)
     ans
 }
-setMethod("duplicated", "Vector", duplicated.Vector)
+setMethod("duplicated", "Vector", .duplicated.Vector)
 
 ### S3/S4 combo for unique.Vector
 unique.Vector <- function(x, incomparables=FALSE, ...)
+    unique(x, incomparables=incomparables, ...)
+.unique.Vector <- function(x, incomparables=FALSE, ...)
 {
     i <- !duplicated(x, incomparables=incomparables, ...)
     extractROWS(x, i)
 }
-setMethod("unique", "Vector", unique.Vector)
+setMethod("unique", "Vector", .unique.Vector)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -169,8 +176,8 @@ setGeneric("countMatches", signature=c("x", "table"),
         standardGeneric("countMatches")
 )
 
-### Problem: using transpose=TRUE generates an invalid Hits object (hits are
-### not ordered):
+### Problem: using transpose=TRUE generates an invalid SortedByQueryHits
+### object (hits are not sorted by query):
 ###   > S4Vectors:::.findAllMatchesInSmallTable(1:6, c(7:5, 4:5),
 ###                                             transpose=TRUE)
 ###   Hits of length 4
@@ -189,7 +196,7 @@ setGeneric("countMatches", signature=c("x", "table"),
     x2 <- match(x, table, ...)
     table2 <- selfmatch(table, ...)
     table_low2high <- reverseSelfmatchMapping(table2)
-    hits_per_x <- table_low2high[x2]
+    hits_per_x <- table_low2high[as.integer(x2)]
     x_hits <- rep.int(seq_along(hits_per_x), sapply_NROW(hits_per_x))
     if (length(x_hits) == 0L) {
         table_hits <- integer(0)
@@ -197,9 +204,9 @@ setGeneric("countMatches", signature=c("x", "table"),
         table_hits <- unlist(hits_per_x, use.names=FALSE)
     }
     if (transpose) {
-        Hits(table_hits, x_hits, length(table), length(x))
+        Hits(table_hits, x_hits, length(table), length(x), sort.by.query=TRUE)
     } else {
-        Hits(x_hits, table_hits, length(x), length(table))
+        Hits(x_hits, table_hits, length(x), length(table), sort.by.query=TRUE)
     }
 }
 
@@ -221,6 +228,16 @@ setMethod("findMatches", c("ANY", "ANY"),
     }
 )
 
+setMethod("findMatches", c("ANY", "missing"),
+    function(x, table, select=c("all", "first", "last"), ...)
+    {
+        ans <- callGeneric(x, x, select=select, ...)
+        if (!is(ans, "Hits"))  # e.g. if 'select' is "first"
+            return(ans)
+        as(ans, "SortedByQuerySelfHits")
+    }
+)
+
 ### Default "countMatches" method. Args in ... are propagated to match() and
 ### selfmatch().
 .countMatches.default <- function(x, table, ...)
@@ -259,14 +276,18 @@ setMethod("countMatches", c("ANY", "ANY"), .countMatches.default)
     extractROWS(x, i)
 }
 sort.Vector <- function(x, decreasing=FALSE, ...)
-    .sort.Vector(x, decreasing=decreasing, ...)
-setMethod("sort", "Vector", sort.Vector)
+    sort(x, decreasing=decreasing, ...)
+setMethod("sort", "Vector", .sort.Vector)
 
 formulaAsListCall <- function(formula) attr(terms(formula), "variables")
 
+formulaValues <- function(x, formula) {
+    eval(formulaAsListCall(formula), as.env(x, environment(formula)))
+}
+
 orderBy <- function(formula, x, decreasing=FALSE, na.last=TRUE) {
-  terms <- eval(formulaAsListCall(formula), as.env(x, environment(formula)))
-  do.call(order, c(decreasing=decreasing, na.last=na.last, terms))
+  values <- formulaValues(x, formula)
+  do.call(order, c(decreasing=decreasing, na.last=na.last, values))
 }
 
 setMethod("xtfrm", "Vector", function(x) {
@@ -360,3 +381,10 @@ setMethod("table", "Vector",
     }
 )
 
+setMethod("xtabs", signature(data = "Vector"),
+          function(formula = ~., data, subset, na.action, exclude = c(NA, NaN),
+                   drop.unused.levels = FALSE)
+{
+    data <- as.env(data, environment(formula), tform=decode)
+    callGeneric()
+})
