@@ -59,13 +59,24 @@ setMethod("aggregate", "ts", stats:::aggregate.ts)
 ### S3/S4 combo for aggregate.Vector
 aggregate.Vector <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
                              frequency=NULL, delta=NULL, ..., simplify=TRUE)
+{
+    aggregate(x, by, FUN, start, end, width, frequency, delta, ...,
+              simplify=simplify)
+}
+
+.aggregate.Vector <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
+                              frequency=NULL, delta=NULL, ..., simplify=TRUE)
 {
     FUN <- match.fun(FUN)
     if (!missing(by)) {
         if (is.list(by)) {
             ans <- aggregate(as.data.frame(x), by=by, FUN=FUN, ...,
                              simplify=simplify)
-            return(ans)
+            return(DataFrame(ans))
+        } else if (is(by, "formula")) {
+            ans <- aggregate(by, as.env(x, environment(by), tform=decode),
+                             FUN=FUN, ...)
+            return(DataFrame(ans))
         }
         start <- structure(start(by), names=names(by))
         end <- end(by)
@@ -89,7 +100,7 @@ aggregate.Vector <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
         indices <- structure(seq_len(n), names = names(end))
     if (is.null(frequency) && is.null(delta)) {
         sapply(indices, function(i)
-               FUN(window.Vector(x, start = start[i], end = end[i]), ...),
+               FUN(Vector_window(x, start = start[i], end = end[i]), ...),
                simplify = simplify)
     } else {
         frequency <- rep(frequency, length.out = n)
@@ -101,20 +112,9 @@ aggregate.Vector <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
                simplify = simplify)
     }
 }
-setMethod("aggregate", "Vector", aggregate.Vector)
+setMethod("aggregate", "Vector", .aggregate.Vector)
 
-### FIXME: This "aggregate" method for vector overrides stats::aggregate()
-### on vector without preserving its behavior! For example:
-###
-###   aggregate(c(NA, 12:20), by=list(rep(1:2, 5)), is.unsorted, TRUE)
-###
-### doesn't give the same result if S4Vectors is loaded or not.
-### As a general rule we should not mask base functionalities with our own,
-### and even less when they behave differently.
-setMethod("aggregate", "vector", aggregate.Vector)
-
-### S3/S4 combo for aggregate.Rle
-aggregate.Rle <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
+.aggregate.Rle <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
                           frequency=NULL, delta=NULL, ..., simplify=TRUE)
 {
     FUN <- match.fun(FUN)
@@ -139,22 +139,10 @@ aggregate.Rle <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
     else
         indices <- structure(seq_len(n), names = names(end))
     if (is.null(frequency) && is.null(delta)) {
-        info <- getStartEndRunAndOffset(x, start, end)
-        runStart <- info[["start"]][["run"]]
-        offsetStart <- info[["start"]][["offset"]]
-        runEnd <- info[["end"]][["run"]]
-        offsetEnd <- info[["end"]][["offset"]]
-        ## Performance Optimization
-        ## Use a stripped down loop with empty Rle object
-        newRle <- new(class(x))
-        sapply(indices,
-               function(i)
-               FUN(.Call2("Rle_window",
-                          x, runStart[i], runEnd[i],
-                          offsetStart[i], offsetEnd[i],
-                          newRle, PACKAGE = "S4Vectors"),
-                   ...),
-               simplify = simplify)
+        width <- end - start + 1L
+        rle_list <- extract_ranges_from_Rle(x, start, width, as.list=TRUE)
+        names(rle_list) <- names(indices)
+        sapply(rle_list, FUN, ..., simplify = simplify)
     } else {
         frequency <- rep(frequency, length.out = n)
         delta <- rep(delta, length.out = n)
@@ -166,18 +154,15 @@ aggregate.Rle <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
                simplify = simplify)
     }
 }
-setMethod("aggregate", "Rle", aggregate.Rle)
+setMethod("aggregate", "Rle", .aggregate.Rle)
 
-### S3/S4 combo for aggregate.List
-aggregate.List <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
+.aggregate.List <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
                            frequency=NULL, delta=NULL, ..., simplify=TRUE)
 {
     if (missing(by)
      || !requireNamespace("IRanges", quietly=TRUE)
      || !is(by, "RangesList")) {
-        ans <- aggregate.Vector(
-                   x, by, FUN, start=start, end=end, width=width,
-                   frequency=frequency, delta=delta, ..., simplify=simplify)
+        ans <- callNextMethod()
         return(ans)
     }
     if (length(x) != length(by))
@@ -188,10 +173,7 @@ aggregate.List <- function(x, by, FUN, start=NULL, end=NULL, width=NULL,
                          aggregate(y[[i]], by = by[[i]], FUN = FUN,
                                    frequency = frequency, delta = delta,
                                    ..., simplify = simplify))
-    ans <- try(SimpleAtomicList(result), silent = TRUE)
-    if (inherits(ans, "try-error"))
-        ans <- new_SimpleList_from_list("SimpleList", result)
-    ans
+    as(result, "List")
 }
-setMethod("aggregate", "List", aggregate.List)
+setMethod("aggregate", "List", .aggregate.List)
 
