@@ -1,7 +1,10 @@
 ### =========================================================================
-### Some low-level (not exported) utility functions to operate on integer
-### vectors
+### Some low-level utility functions to operate on integer vectors
 ### -------------------------------------------------------------------------
+###
+### Unless stated otherwise, the functions defined in this file are not
+### exported.
+###
 
 
 anyMissingOrOutside <- function(x, lower = -.Machine$integer.max,
@@ -49,65 +52,18 @@ diffWithLast <- function(x, last)
   .Call2("Integer_diff_with_last", x, last, PACKAGE="S4Vectors")
 }
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Fast ordering of an integer vector.
-###
-
-### We want this ordering to be "stable".
-orderInteger <- function(x, decreasing=FALSE, na.last=NA)
-{
-    if (is.factor(x)) {
-        input_is_factor <- TRUE
-        x_delta <- length(levels(x)) - 1L
-        x <- as.integer(x)
-    } else {
-        if (!is.integer(x))
-            stop("'x' must be an integer vector or a factor")
-        input_is_factor <- FALSE
-    }
-    x_min <- suppressWarnings(min(x, na.rm=TRUE))
-    if (x_min == Inf) {
-        if (is.na(na.last))
-            return(integer(0))
-        else
-            return(seq_len(length(x)))
-    }
-    ## At this point 'x' is guaranteed to contain at least one non NA value.
-    has_NAs <- anyNA(x)
-    if (!has_NAs && !decreasing && isStrictlySorted(x)) {
-        return(seq_along(x))
-    }
-    if (!input_is_factor)
-        x_delta <- max(x, na.rm=TRUE) - x_min
-    if (x_delta < 100000L) {
-        if (!has_NAs) {
-            na.last <- TRUE
-        }
-        ## "radix" method is stable.
-        return(sort.list(x, decreasing=decreasing, na.last=na.last,
-                         method="radix"))
-    }
-    
-    if (!has_NAs || is.na(na.last)) {
-        if (has_NAs)
-            x <- x[!is.na(x)]
-        ## Uses _get_order_of_int_array() at the C level which is stable.
-        return(.Call2("Integer_order", x, decreasing, PACKAGE="S4Vectors"))
-    }
-    ## At this point 'x' has NAs and we must keep them ('na.last' is not NA).
-    ## We can't use sort.list() with method="quick" or method="shell" here
-    ## because they are not stable algorithms (and in addition method="quick"
-    ## is only supported when 'na.last' is NA). So we use order() with an
-    ## extra vector to break ties, which is a trick to make it stable.
-    ## Unfortunately this is very inefficient (about twice slower than
-    ## using sort.list() with method="shell").
-    ## TODO: Modify .Call entry point Integer_order to support 'na.last' arg.
-    if (decreasing)
-        y <- length(x):1L
-    else
-        y <- seq_len(length(x))
-    order(x, y, decreasing=decreasing, na.last=na.last)
+### x: integer vector.
+### breakpoints: vector of positions on 'x' in increasing order.
+### Equivalent to (but 10x faster than):
+###     sum(relist(x, PartitioningByEnd(breakpoints)))
+### Also equivalent to (but 200x faster than):
+###     f <- rep(factor(seq_along(breakpoints)), diff(c(0L, breakpoints)))
+###     vapply(split(x, f, drop=FALSE), sum, integer(1), USE.NAMES=FALSE)
+groupsum <- function(x, breakpoints)
+{   
+    if (last_or(breakpoints, 0L) != length(x))
+        stop("invalid 'breakpoints' argument")
+    diffWithInitialZero(cumsum(x)[breakpoints])
 }
 
 
@@ -139,7 +95,7 @@ orderInteger <- function(x, decreasing=FALSE, na.last=NA)
     method
 }
 
-compareIntegerPairs <- function(a1, b1, a2, b2)
+pcompareIntegerPairs <- function(a1, b1, a2, b2)
 {
     a1 <- .normargIntegerOrFactor(a1, "a1")
     b1 <- .normargIntegerOrFactor(b1, "b1")
@@ -149,7 +105,7 @@ compareIntegerPairs <- function(a1, b1, a2, b2)
     b2 <- .normargIntegerOrFactor(b2, "b2")
     if (length(a2) != length(b2))
         stop("'a2' and 'b2' must have the same length")
-    .Call2("Integer_compare2", a1, b1, a2, b2, PACKAGE="S4Vectors")
+    .Call2("Integer_pcompare2", a1, b1, a2, b2, PACKAGE="S4Vectors")
 }
 
 sortedIntegerPairs <- function(a, b, decreasing=FALSE, strictly=FALSE)
@@ -159,33 +115,13 @@ sortedIntegerPairs <- function(a, b, decreasing=FALSE, strictly=FALSE)
     .Call2("Integer_sorted2", a, b, decreasing, strictly, PACKAGE="S4Vectors")
 }
 
-### For 'a' and 'b' integer vectors of equal length with no NAs,
-### 'orderIntegerPairs(a, b)' is equivalent to (but faster than):
-###
-###   order(a, b)
-###
-### Benchmarking:
-###
-###   # Generating random pairs (representing ranges).
-###   library(S4Vectors)
-###   N <- 20000000L  # nb of ranges
-###   W <- 40L        # average width of the ranges
-###   max_end <- 55000000L
-###   set.seed(777)
-###   a <- sample(max_end - W - 2L, N, replace=TRUE)
-###   b <- W + sample(-3:3, N, replace=TRUE)
-###   ## Takes < 10 sec.:
-###   oo <- S4Vectors:::orderIntegerPairs(a, b)
-###   ## Takes about 1 min.:
-###   oo2 <- order(a, b)
-###   identical(oo, oo2)  # TRUE
-###
-### For efficiency reasons, we don't support (and don't even check) for NAs.
+### Exported!
 orderIntegerPairs <- function(a, b, decreasing=FALSE)
 {
     a <- .normargIntegerOrFactor(a, "a")
     b <- .normargIntegerOrFactor(b, "b")
-    .Call2("Integer_order2", a, b, decreasing, PACKAGE="S4Vectors")
+    #.Call2("Integer_order2", a, b, decreasing, PACKAGE="S4Vectors")
+    base::order(a, b, decreasing=decreasing)
 }
 
 .matchIntegerPairs_quick <- function(a1, b1, a2, b2, nomatch=NA_integer_)
@@ -202,6 +138,7 @@ orderIntegerPairs <- function(a, b, decreasing=FALSE)
            PACKAGE="S4Vectors")
 }
 
+### Exported!
 matchIntegerPairs <- function(a1, b1, a2, b2, nomatch=NA_integer_,
                               method=c("auto", "quick", "hash"))
 {
@@ -237,6 +174,7 @@ matchIntegerPairs <- function(a1, b1, a2, b2, nomatch=NA_integer_,
     .Call2("Integer_selfmatch2_hash", a, b, PACKAGE="S4Vectors")
 }
 
+### Exported!
 selfmatchIntegerPairs <- function(a, b, method=c("auto", "quick", "hash"))
 {
     a <- .normargIntegerOrFactor(a, "a")
@@ -252,6 +190,8 @@ selfmatchIntegerPairs <- function(a, b, method=c("auto", "quick", "hash"))
     ans
 }
 
+### Exported!
+###
 ### For 'a' and 'b' integer vectors of equal length with no NAs,
 ### 'duplicatedIntegerPairs(a, b)' is equivalent to (but much faster than):
 ###
@@ -315,19 +255,25 @@ runEndsOfIntegerPairs <- function(a, b)
 ### Fast ordering/comparing of integer quadruplets.
 ###
 
-### For 'a', 'b', 'c' and 'd' integer vectors of equal length with no NAs,
-### 'orderIntegerQuads(a, b, c, d)' is equivalent to (but faster than):
-###
-###   order(a, b, c, d)
-###
-### For efficiency reasons, we don't support (and don't even check) for NAs.
+sortedIntegerQuads <- function(a, b, c, d, decreasing=FALSE, strictly=FALSE)
+{
+    a <- .normargIntegerOrFactor(a, "a")
+    b <- .normargIntegerOrFactor(b, "b")
+    c <- .normargIntegerOrFactor(c, "c")
+    d <- .normargIntegerOrFactor(d, "d")
+    .Call2("Integer_sorted4", a, b, c, d, decreasing, strictly,
+           PACKAGE="S4Vectors")
+}
+
+### Exported!
 orderIntegerQuads <- function(a, b, c, d, decreasing=FALSE)
 {
     a <- .normargIntegerOrFactor(a, "a")
     b <- .normargIntegerOrFactor(b, "b")
     c <- .normargIntegerOrFactor(c, "c")
     d <- .normargIntegerOrFactor(d, "d")
-    .Call2("Integer_order4", a, b, c, d, decreasing, PACKAGE="S4Vectors")
+    #.Call2("Integer_order4", a, b, c, d, decreasing, PACKAGE="S4Vectors")
+    base::order(a, b, c, d, decreasing=decreasing)
 }
 
 .matchIntegerQuads_quick <- function(a1, b1, c1, d1, a2, b2, c2, d2,
@@ -346,6 +292,7 @@ orderIntegerQuads <- function(a, b, c, d, decreasing=FALSE)
            PACKAGE="S4Vectors")
 }
 
+### Exported!
 matchIntegerQuads <- function(a1, b1, c1, d1, a2, b2, c2, d2,
                               nomatch=NA_integer_,
                               method=c("auto", "quick", "hash"))
@@ -391,6 +338,7 @@ matchIntegerQuads <- function(a1, b1, c1, d1, a2, b2, c2, d2,
     .Call2("Integer_selfmatch4_hash", a, b, c, d, PACKAGE="S4Vectors")
 }
 
+### Exported!
 selfmatchIntegerQuads <- function(a, b, c, d,
                                   method=c("auto", "quick", "hash"))
 {
@@ -411,6 +359,7 @@ selfmatchIntegerQuads <- function(a, b, c, d,
     ans
 }
 
+### Exported!
 duplicatedIntegerQuads <- function(a, b, c, d,
                                    fromLast=FALSE,
                                    method=c("auto", "quick", "hash"))
