@@ -1,6 +1,7 @@
 #include "S4Vectors.h"
 
-#include <limits.h> /* for INT_MAX */
+#include <stdlib.h>  /* for malloc(), free() */
+#include <limits.h>  /* for INT_MAX */
 
 
 static int get_bucket_idx_for_int_pair(const struct htab *htab,
@@ -143,6 +144,7 @@ SEXP Integer_diff_with_0(SEXP x)
 	return ans;
 }
 
+
 /****************************************************************************
  * --- .Call ENTRY POINT ---
  * diff(c(x, last))
@@ -167,9 +169,41 @@ SEXP Integer_diff_with_last(SEXP x, SEXP last)
   return ans;
 }
 
+
 /****************************************************************************
+ * Fast ordering of an integer vector.
+ */
+
+/* --- .Call ENTRY POINT --- */
+SEXP Integer_order(SEXP x, SEXP decreasing, SEXP use_radix)
+{
+	int ans_len, i, *ans_elt_p;
+	SEXP ans;
+
+	if (LENGTH(decreasing) != 1)
+		error("S4Vectors internal error in Integer_order(): "
+		      "'decreasing' must be of length 1");
+	ans_len = LENGTH(x);
+	PROTECT(ans = NEW_INTEGER(ans_len));
+	for (i = 1, ans_elt_p = INTEGER(ans); i <= ans_len; i++, ans_elt_p++)
+		*ans_elt_p = i;
+	i = _sort_ints(INTEGER(ans), ans_len,
+		       INTEGER(x) - 1,
+		       LOGICAL(decreasing)[0],
+		       LOGICAL(use_radix)[0], NULL, NULL);
+	UNPROTECT(1);
+	if (i != 0)
+		error("S4Vectors internal error in Integer_order(): "
+		      "memory allocation failed");
+	return ans;
+}
+
+
+/****************************************************************************
+ * Fast ordering/comparing of integer pairs.
+ *
  * The .Call entry points in this section are the workhorses behind
- * orderInteger(), orderIntegerPairs(), matchIntegerPairs(), and
+ * sortedIntegerPairs(), orderIntegerPairs(), matchIntegerPairs(), and
  * duplicatedIntegerPairs().
  */
 
@@ -196,20 +230,6 @@ int _check_integer_pairs(SEXP a, SEXP b,
 	return len;
 }
 
-/* --- .Call ENTRY POINT --- */
-SEXP Integer_order(SEXP x, SEXP decreasing)
-{
-	int ans_length;
-	SEXP ans;
-
-	ans_length = LENGTH(x);
-	PROTECT(ans = NEW_INTEGER(ans_length));
-	_get_order_of_int_array(INTEGER(x), ans_length,
-				LOGICAL(decreasing)[0], INTEGER(ans), 1);
-	UNPROTECT(1);
-	return ans;
-}
-
 /* --- .Call ENTRY POINT ---
  * 'a1' and 'b1': integer vectors of the same length M.
  * 'a2' and 'b2': integer vectors of the same length N.
@@ -219,7 +239,7 @@ SEXP Integer_order(SEXP x, SEXP decreasing)
  * object, except if M or N is 0 in which case the object with length != 0 is
  * truncated to length 0.
  */
-SEXP Integer_compare2(SEXP a1, SEXP b1, SEXP a2, SEXP b2)
+SEXP Integer_pcompare2(SEXP a1, SEXP b1, SEXP a2, SEXP b2)
 {
 	int npair1, npair2, ans_len;
 	const int *a1_p, *b1_p, *a2_p, *b2_p;
@@ -232,8 +252,8 @@ SEXP Integer_compare2(SEXP a1, SEXP b1, SEXP a2, SEXP b2)
 	else
 		ans_len = npair1 >= npair2 ? npair1 : npair2;
 	PROTECT(ans = NEW_INTEGER(ans_len));
-	_compare_int_pairs(a1_p, b1_p, npair1, a2_p, b2_p, npair2,
-			   INTEGER(ans), ans_len, 1);
+	_pcompare_int_pairs(a1_p, b1_p, npair1, a2_p, b2_p, npair2,
+			    INTEGER(ans), ans_len, 1);
 	UNPROTECT(1);
 	return ans;
 }
@@ -252,17 +272,28 @@ SEXP Integer_sorted2(SEXP a, SEXP b, SEXP decreasing, SEXP strictly)
 }
 
 /* --- .Call ENTRY POINT --- */
-SEXP Integer_order2(SEXP a, SEXP b, SEXP decreasing)
+SEXP Integer_order2(SEXP a, SEXP b, SEXP decreasing, SEXP use_radix)
 {
-	int ans_length;
+	int ans_len, i, *ans_elt_p;
 	const int *a_p, *b_p;
 	SEXP ans;
 
-	ans_length = _check_integer_pairs(a, b, &a_p, &b_p, "a", "b");
-	PROTECT(ans = NEW_INTEGER(ans_length));
-	_get_order_of_int_pairs(a_p, b_p, ans_length,
-				LOGICAL(decreasing)[0], INTEGER(ans), 1);
+	if (LENGTH(decreasing) != 2)
+		error("S4Vectors internal error in Integer_order2(): "
+		      "'decreasing' must be of length 2");
+	ans_len = _check_integer_pairs(a, b, &a_p, &b_p, "a", "b");
+	PROTECT(ans = NEW_INTEGER(ans_len));
+	for (i = 1, ans_elt_p = INTEGER(ans); i <= ans_len; i++, ans_elt_p++)
+		*ans_elt_p = i;
+	i = _sort_int_pairs(INTEGER(ans), ans_len,
+			    a_p - 1, b_p - 1,
+			    LOGICAL(decreasing)[0],
+			    LOGICAL(decreasing)[1],
+			    LOGICAL(use_radix)[0], NULL, NULL);
 	UNPROTECT(1);
+	if (i != 0)
+		error("S4Vectors internal error in Integer_order2(): "
+		      "memory allocation failed");
 	return ans;
 }
 
@@ -278,8 +309,8 @@ SEXP Integer_match2_quick(SEXP a1, SEXP b1, SEXP a2, SEXP b2, SEXP nomatch)
 	nomatch0 = INTEGER(nomatch)[0];
 	o1 = (int *) R_alloc(sizeof(int), len1);
 	o2 = (int *) R_alloc(sizeof(int), len2);
-	_get_order_of_int_pairs(a1_p, b1_p, len1, 0, o1, 0);
-	_get_order_of_int_pairs(a2_p, b2_p, len2, 0, o2, 0);
+	_get_order_of_int_pairs(a1_p, b1_p, len1, 0, 0, o1, 0);
+	_get_order_of_int_pairs(a2_p, b2_p, len2, 0, 0, o2, 0);
 	PROTECT(ans = NEW_INTEGER(len1));
 	_get_matches_of_ordered_int_pairs(a1_p, b1_p, o1, len1,
 					  a2_p, b2_p, o2, len2,
@@ -297,7 +328,7 @@ SEXP Integer_selfmatch2_quick(SEXP a, SEXP b)
 
 	len = _check_integer_pairs(a, b, &a_p, &b_p, "a", "b");
 	o1 = (int *) R_alloc(sizeof(int), len);
-	_get_order_of_int_pairs(a_p, b_p, len, 0, o1, 0);
+	_get_order_of_int_pairs(a_p, b_p, len, 0, 0, o1, 0);
 	PROTECT(ans = NEW_INTEGER(len));
 	_get_matches_of_ordered_int_pairs(a_p, b_p, o1, len,
 					  a_p, b_p, o1, len,
@@ -344,16 +375,16 @@ SEXP Integer_match2_hash(SEXP a1, SEXP b1, SEXP a2, SEXP b2, SEXP nomatch)
 /* --- .Call ENTRY POINT --- */
 SEXP Integer_selfmatch2_hash(SEXP a, SEXP b)
 {
-	int ans_length, *ans0, i, bucket_idx, i2;
+	int ans_len, *ans0, i, bucket_idx, i2;
 	const int *a_p, *b_p;
 	struct htab htab;
 	SEXP ans;
 
-	ans_length = _check_integer_pairs(a, b, &a_p, &b_p, "a", "b");
-	htab = _new_htab(ans_length);
-	PROTECT(ans = NEW_INTEGER(ans_length));
+	ans_len = _check_integer_pairs(a, b, &a_p, &b_p, "a", "b");
+	htab = _new_htab(ans_len);
+	PROTECT(ans = NEW_INTEGER(ans_len));
 	ans0 = INTEGER(ans);
-	for (i = 0; i < ans_length; i++) {
+	for (i = 0; i < ans_len; i++) {
 		bucket_idx = get_bucket_idx_for_int_pair(&htab,
 					a_p[i], b_p[i],
 					a_p, b_p);
@@ -371,8 +402,11 @@ SEXP Integer_selfmatch2_hash(SEXP a, SEXP b)
 
 
 /****************************************************************************
+ * Fast ordering/comparing of integer quadruplets.
+ *
  * The .Call entry points in this section are the workhorses behind
- * orderIntegerQuads(), matchIntegerQuads() and duplicatedIntegerQuads().
+ * sortedIntegerQuads(), orderIntegerQuads(), matchIntegerQuads(), and
+ * duplicatedIntegerQuads().
  */
 
 /*
@@ -404,19 +438,49 @@ int _check_integer_quads(SEXP a, SEXP b, SEXP c, SEXP d,
 }
 
 /* --- .Call ENTRY POINT --- */
-SEXP Integer_order4(SEXP a, SEXP b, SEXP c, SEXP d, SEXP decreasing)
+SEXP Integer_sorted4(SEXP a, SEXP b, SEXP c, SEXP d,
+		     SEXP decreasing, SEXP strictly)
 {
-	int ans_length;
+	const int *a_p, *b_p, *c_p, *d_p;
+	int nquad, ans;
+
+	nquad = _check_integer_quads(a, b, c, d,
+				     &a_p, &b_p, &c_p, &d_p,
+				     "a", "b", "c", "d");
+	ans = _int_quads_are_sorted(a_p, b_p, c_p, d_p, nquad,
+				    LOGICAL(decreasing)[0],
+				    LOGICAL(strictly)[0]);
+	return ScalarLogical(ans);
+}
+
+/* --- .Call ENTRY POINT --- */
+SEXP Integer_order4(SEXP a, SEXP b, SEXP c, SEXP d,
+		    SEXP decreasing, SEXP use_radix)
+{
+	int ans_len, i, *ans_elt_p;
 	const int *a_p, *b_p, *c_p, *d_p;
 	SEXP ans;
 
-	ans_length = _check_integer_quads(a, b, c, d,
-					  &a_p, &b_p, &c_p, &d_p,
-					  "a", "b", "c", "d");
-	PROTECT(ans = NEW_INTEGER(ans_length));
-	_get_order_of_int_quads(a_p, b_p, c_p, d_p, ans_length,
-				LOGICAL(decreasing)[0], INTEGER(ans), 1);
+	if (LENGTH(decreasing) != 4)
+		error("S4Vectors internal error in Integer_order4(): "
+		      "'decreasing' must be of length 4");
+	ans_len = _check_integer_quads(a, b, c, d,
+				       &a_p, &b_p, &c_p, &d_p,
+				       "a", "b", "c", "d");
+	PROTECT(ans = NEW_INTEGER(ans_len));
+	for (i = 1, ans_elt_p = INTEGER(ans); i <= ans_len; i++, ans_elt_p++)
+		*ans_elt_p = i;
+	i = _sort_int_quads(INTEGER(ans), ans_len,
+			    a_p - 1, b_p - 1, c_p - 1, d_p - 1,
+			    LOGICAL(decreasing)[0],
+			    LOGICAL(decreasing)[1],
+			    LOGICAL(decreasing)[2],
+			    LOGICAL(decreasing)[3],
+			    LOGICAL(use_radix)[0], NULL, NULL);
 	UNPROTECT(1);
+	if (i != 0)
+		error("S4Vectors internal error in Integer_order4(): "
+		      "memory allocation failed");
 	return ans;
 }
 
@@ -437,8 +501,10 @@ SEXP Integer_match4_quick(SEXP a1, SEXP b1, SEXP c1, SEXP d1,
 	nomatch0 = INTEGER(nomatch)[0];
 	o1 = (int *) R_alloc(sizeof(int), len1);
 	o2 = (int *) R_alloc(sizeof(int), len2);
-	_get_order_of_int_quads(a1_p, b1_p, c1_p, d1_p, len1, 0, o1, 0);
-	_get_order_of_int_quads(a2_p, b2_p, c2_p, d2_p, len2, 0, o2, 0);
+	_get_order_of_int_quads(a1_p, b1_p, c1_p, d1_p, len1,
+				0, 0, 0, 0, o1, 0);
+	_get_order_of_int_quads(a2_p, b2_p, c2_p, d2_p, len2,
+				0, 0, 0, 0, o2, 0);
 	PROTECT(ans = NEW_INTEGER(len1));
 	_get_matches_of_ordered_int_quads(a1_p, b1_p, c1_p, d1_p, o1, len1,
 					  a2_p, b2_p, c2_p, d2_p, o2, len2,
@@ -458,7 +524,7 @@ SEXP Integer_selfmatch4_quick(SEXP a, SEXP b, SEXP c, SEXP d)
 				   &a_p, &b_p, &c_p, &d_p,
 				   "a", "b", "c", "d");
 	o1 = (int *) R_alloc(sizeof(int), len);
-	_get_order_of_int_quads(a_p, b_p, c_p, d_p, len, 0, o1, 0);
+	_get_order_of_int_quads(a_p, b_p, c_p, d_p, len, 0, 0, 0, 0, o1, 0);
 	PROTECT(ans = NEW_INTEGER(len));
 	_get_matches_of_ordered_int_quads(a_p, b_p, c_p, d_p, o1, len,
 					  a_p, b_p, c_p, d_p, o1, len,
@@ -510,18 +576,18 @@ SEXP Integer_match4_hash(SEXP a1, SEXP b1, SEXP c1, SEXP d1,
 /* --- .Call ENTRY POINT --- */
 SEXP Integer_selfmatch4_hash(SEXP a, SEXP b, SEXP c, SEXP d)
 {
-	int ans_length, *ans0, i, bucket_idx, i2;
+	int ans_len, *ans0, i, bucket_idx, i2;
 	const int *a_p, *b_p, *c_p, *d_p;
 	struct htab htab;
 	SEXP ans;
 
-	ans_length = _check_integer_quads(a, b, c, d,
-					  &a_p, &b_p, &c_p, &d_p,
-					  "a", "b", "c", "d");
-	htab = _new_htab(ans_length);
-	PROTECT(ans = NEW_INTEGER(ans_length));
+	ans_len = _check_integer_quads(a, b, c, d,
+				       &a_p, &b_p, &c_p, &d_p,
+				       "a", "b", "c", "d");
+	htab = _new_htab(ans_len);
+	PROTECT(ans = NEW_INTEGER(ans_len));
 	ans0 = INTEGER(ans);
-	for (i = 0; i < ans_length; i++) {
+	for (i = 0; i < ans_len; i++) {
 		bucket_idx = get_bucket_idx_for_int_quad(&htab,
 					a_p[i], b_p[i], c_p[i], d_p[i],
 					a_p, b_p, c_p, d_p);
@@ -700,7 +766,7 @@ SEXP Integer_sorted_merge(SEXP x, SEXP y)
 
 SEXP Integer_mseq(SEXP from, SEXP to)
 {
-	int i, j, n, ans_length, *from_elt, *to_elt, *ans_elt;
+	int i, j, n, ans_len, *from_elt, *to_elt, *ans_elt;
 	SEXP ans;
 
 	if (!IS_INTEGER(from) || !IS_INTEGER(to))
@@ -710,14 +776,14 @@ SEXP Integer_mseq(SEXP from, SEXP to)
 	if (n != LENGTH(to))
 		error("lengths of 'from' and 'to' must be equal");
 
-	ans_length = 0;
+	ans_len = 0;
 	for (i = 0, from_elt = INTEGER(from), to_elt = INTEGER(to); i < n;
 		 i++, from_elt++, to_elt++) {
-		ans_length += (*from_elt <= *to_elt ? *to_elt - *from_elt
-						    : *from_elt - *to_elt) + 1;
+		ans_len += (*from_elt <= *to_elt ? *to_elt - *from_elt
+						 : *from_elt - *to_elt) + 1;
 	}
 
-	PROTECT(ans = NEW_INTEGER(ans_length));
+	PROTECT(ans = NEW_INTEGER(ans_len));
 	ans_elt = INTEGER(ans);
 	for (i = 0, from_elt = INTEGER(from), to_elt = INTEGER(to); i < n;
 		 i++, from_elt++, to_elt++) {
@@ -742,23 +808,23 @@ SEXP Integer_mseq(SEXP from, SEXP to)
 
 SEXP Integer_fancy_mseq(SEXP lengths, SEXP offset, SEXP rev)
 {
-	int lengths_length, offset_length, rev_length, ans_length,
+	int lengths_len, offset_len, rev_len, ans_len,
 	    i, length, *ans_elt, i2, i3, offset_elt, rev_elt, j;
 	const int *lengths_elt;
 	SEXP ans;
 
-	lengths_length = LENGTH(lengths);
-	offset_length = LENGTH(offset);
-	rev_length = LENGTH(rev);
-	if (lengths_length != 0) {
-		if (offset_length == 0)
+	lengths_len = LENGTH(lengths);
+	offset_len = LENGTH(offset);
+	rev_len = LENGTH(rev);
+	if (lengths_len != 0) {
+		if (offset_len == 0)
 			error("'offset' has length 0 but not 'lengths'");
-		if (rev_length == 0)
+		if (rev_len == 0)
 			error("'rev' has length 0 but not 'lengths'");
 	}
-	ans_length = 0;
+	ans_len = 0;
 	for (i = 0, lengths_elt = INTEGER(lengths);
-	     i < lengths_length;
+	     i < lengths_len;
 	     i++, lengths_elt++)
 	{
 		length = *lengths_elt;
@@ -766,17 +832,17 @@ SEXP Integer_fancy_mseq(SEXP lengths, SEXP offset, SEXP rev)
 			error("'lengths' contains NAs");
 		if (length < 0)
 			length = -length;
-		ans_length += length;
+		ans_len += length;
 	}
-	PROTECT(ans = NEW_INTEGER(ans_length));
+	PROTECT(ans = NEW_INTEGER(ans_len));
 	ans_elt = INTEGER(ans);
 	for (i = i2 = i3 = 0, lengths_elt = INTEGER(lengths);
-	     i < lengths_length;
+	     i < lengths_len;
 	     i++, i2++, i3++, lengths_elt++)
 	{
-		if (i2 >= offset_length)
+		if (i2 >= offset_len)
 			i2 = 0; /* recycle */
-		if (i3 >= rev_length)
+		if (i3 >= rev_len)
 			i3 = 0; /* recycle */
 		length = *lengths_elt;
 		offset_elt = INTEGER(offset)[i2];
